@@ -5,6 +5,7 @@
 
 #define instruction_native_t        instruction_arm64_t
 #define dynarec_native_t            dynarec_arm_t
+#define extcache_native_t           neoncache_t
 
 #define ADDITIONNAL_DEFINITION()  \
     int fpuCacheNeedsTransform(dynarec_native_t* dyn, int ninst);
@@ -16,6 +17,7 @@
 #include "arm64/arm64_printer.h"
 #include "arm64/dynarec_arm64_private.h"
 #include "arm64/dynarec_arm64_functions.h"
+#include "arm64/dynarec_arm64_arch.h"
 // Limit here is defined by LD litteral, that is 19bits
 #define MAXBLOCK_SIZE ((1<<19)-200)
 
@@ -23,10 +25,16 @@
 #define UPDATE_SPECIFICS(A)     updateNativeFlags(A)
 #define PREUPDATE_SPECIFICS(A)
 
+#define ARCH_SIZE(A)    get_size_arch(A)
+#define ARCH_FILL(A, B, C) populate_arch(A, B, C)
+#define ARCH_ADJUST(A, B, C, D) adjust_arch(A, B, C, D)
+#define STOP_NATIVE_FLAGS(A, B)   A->insts[B].nat_flags_op = NAT_FLAG_OP_UNUSABLE
+#define ARCH_UNALIGNED(A, B) arch_unaligned(A, B)
 #elif defined(LA64)
 
 #define instruction_native_t        instruction_la64_t
 #define dynarec_native_t            dynarec_la64_t
+#define extcache_native_t           lsxcache_t
 
 #define ADDITIONNAL_DEFINITION() \
     int fpuCacheNeedsTransform(dynarec_native_t* dyn, int ninst);
@@ -42,11 +50,18 @@
 
 #define RAZ_SPECIFIC(A, N)
 #define UPDATE_SPECIFICS(A)
-#define PREUPDATE_SPECIFICS(A)
+#define PREUPDATE_SPECIFICS(A) updateNativeFlags(A)
+
+#define ARCH_SIZE(A)    0
+#define ARCH_FILL(A, B, C)  NULL
+#define ARCH_ADJUST(A, B, C, D) {}
+#define STOP_NATIVE_FLAGS(A, B) {}
+#define ARCH_UNALIGNED(A, B) 0
 #elif defined(RV64)
 
 #define instruction_native_t        instruction_rv64_t
 #define dynarec_native_t            dynarec_rv64_t
+#define extcache_native_t           extcache_t
 
 #define ADDITIONNAL_DEFINITION()                                  \
     int fpuCacheNeedsTransform(dynarec_native_t* dyn, int ninst); \
@@ -59,12 +74,19 @@
 #include "rv64/rv64_printer.h"
 #include "rv64/dynarec_rv64_private.h"
 #include "rv64/dynarec_rv64_functions.h"
+#include "rv64/dynarec_rv64_arch.h"
 // Limit here is unconditionnal jump, that is signed 21bits
 #define MAXBLOCK_SIZE ((1<<20)-200)
 
 #define RAZ_SPECIFIC(A, N)
 #define UPDATE_SPECIFICS(A)
 #define PREUPDATE_SPECIFICS(A) updateNativeFlags(A)
+
+#define ARCH_SIZE(A)    get_size_arch(A)
+#define ARCH_FILL(A, B, C) populate_arch(A, B, C)
+#define ARCH_ADJUST(A, B, C, D) {}
+#define STOP_NATIVE_FLAGS(A, B) {}
+#define ARCH_UNALIGNED(A, B) arch_unaligned(A, B)
 #else
 #error Unsupported platform
 #endif
