@@ -39,7 +39,7 @@ typedef struct instruction_x64_s {
     int         jmp_insts;  // instuction to jump to (-1 if out of the block)
     uint8_t     jmp_cond:1;   // 1 of conditionnal jump
     uint8_t     has_next:1;   // does this opcode can continue to the next?
-    uint8_t     has_callret:1;    // this instruction have an optimised call setup
+    uint8_t     has_callret:1;    // this instruction have an optimized call setup
     uint8_t     alive:1;    // this opcode gets executed (0 if dead code in that block)
     uint8_t     barrier;    // next instruction is a jump point, so no optim allowed
     uint8_t     state_flags;// One of SF_XXX state
@@ -51,6 +51,4 @@ typedef struct instruction_x64_s {
     uint8_t     need_after; // calculated
 } instruction_x64_t;
 
-void printf_x64_instruction(zydis_dec_t* dec, instruction_x64_t* inst, const char* name);
-
 #endif //__DYNAREC_PRIVATE_H_
