@@ -86,6 +86,8 @@ static uint8_t Peek8(uintptr_t addr, uintptr_t offset)
     return *(uint8_t*)(addr+offset);
 }
 
+void x64Print(x64emu_t* emu, char* buff, size_t buffsz, const char* func, int tid, wrapper_t w);
+
 void x64Int3(x64emu_t* emu, uintptr_t* addr)
 {
     if(box64_is32bits) {
@@ -106,21 +108,21 @@ void x64Int3(x64emu_t* emu, uintptr_t* addr)
             wrapper_t w = bridge->w;
             a = F64(addr);
             R_RIP = *addr;
-            /* This party can be used to trace only 1 specific lib (but it is quite slow)
+            /* This part can be used to trace only 1 specific lib (but it is quite slow)
             elfheader_t *h = FindElfAddress(my_context, *(uintptr_t*)(R_ESP));
             int have_trace = 0;
             if(h && strstr(ElfName(h), "libMiles")) have_trace = 1;*/
-            if(box64_log>=LOG_DEBUG || cycle_log) {
+            if(BOX64ENV(log)>=LOG_DEBUG || BOX64ENV(rolling_log)) {
                 int tid = GetTID();
                 char t_buff[256] = "\0";
                 char buff2[64] = "\0";
                 char buff3[64] = "\0";
                 int cycle_line = my_context->current_line;
-                if(cycle_log) {
-                    my_context->current_line = (my_context->current_line+1)%cycle_log;
+                if(BOX64ENV(rolling_log)) {
+                    my_context->current_line = (my_context->current_line+1)%BOX64ENV(rolling_log);
                 }
-                char* buff = cycle_log?my_context->log_call[cycle_line]:t_buff;
-                char* buffret = cycle_log?my_context->log_ret[cycle_line]:NULL;
+                char* buff = BOX64ENV(rolling_log)?my_context->log_call[cycle_line]:t_buff;
+                char* buffret = BOX64ENV(rolling_log)?my_context->log_ret[cycle_line]:NULL;
                 if(buffret) buffret[0] = '\0';
                 char *tmp;
                 int post = 0;
@@ -132,7 +134,7 @@ void x64Int3(x64emu_t* emu, uintptr_t* addr)
                 if(!s)
                     s = GetNativeName((void*)a);
                 if(a==(uintptr_t)PltResolver64) {
-                    if(cycle_log) {
+                    if(BOX64ENV(rolling_log)) {
                         uintptr_t addr = *((uint64_t*)(R_RSP));
                         int slot = *((uint64_t*)(R_RSP+8));
                         elfheader_t *h = (elfheader_t*)addr;
@@ -318,11 +320,11 @@ void x64Int3(x64emu_t* emu, uintptr_t* addr)
                     post = 7;
                     snprintf(buff, 256, "%04d|%p: Calling %s(%p, 0x%X)", tid, *(void**)(R_RSP), s, (void*)R_RDI, R_ESI);
                 } else {
-                    snprintf(buff, 256, "%04d|%p: Calling %s(0x%lX, 0x%lX, 0x%lX, ...)", tid, *(void**)(R_RSP), s, R_RDI, R_RSI, R_RDX);
+                    x64Print(emu, buff, 256, s, tid, w);
                 }
-                if(!cycle_log) {
+                if(!BOX64ENV(rolling_log)) {
                     mutex_lock(&emu->context->mutex_trace);
-                    printf_log(LOG_NONE, "%s =>", buff);
+                    printf_log_prefix(0, LOG_NONE, "%s =>", buff);
                     mutex_unlock(&emu->context->mutex_trace);
                 }
                 w(emu, a);   // some function never come back, so unlock the mutex first!
@@ -372,11 +374,11 @@ void x64Int3(x64emu_t* emu, uintptr_t* addr)
                 else if(perr==3 && (S_RAX)==-1)
                     snprintf(buff3, 64, " (errno=%d:\"%s\")", errno, strerror(errno));
 
-                if(cycle_log)
+                if(BOX64ENV(rolling_log))
                     snprintf(buffret, 128, "0x%lX%s%s", R_RAX, buff2, buff3);
                 else {
                     mutex_lock(&emu->context->mutex_trace);
-                    printf_log(LOG_NONE, " return 0x%lX%s%s\n", R_RAX, buff2, buff3);
+                    printf_log_prefix(0, LOG_NONE, " return 0x%lX%s%s\n", R_RAX, buff2, buff3);
                     mutex_unlock(&emu->context->mutex_trace);
                 }
             } else
@@ -389,7 +391,7 @@ void x64Int3(x64emu_t* emu, uintptr_t* addr)
         printf_log(LOG_DEBUG, "%04d|Warning, x64int3 with no CC opcode at %p?\n", GetTID(), (void*)R_RIP);
         return;
     }
-    if(!box64_ignoreint3 && my_context->signals[SIGTRAP]) {
+    if(!BOX64ENV(ignoreint3) && my_context->signals[SIGTRAP]) {
         R_RIP = *addr;  // update RIP
         emit_signal(emu, SIGTRAP, NULL, 3);
     } else {
@@ -404,12 +406,12 @@ int GetTID()
     return syscall(SYS_gettid);
 }
 
-void print_cycle_log(int loglevel) {
-    if(cycle_log) {
+void print_rolling_log(int loglevel) {
+    if(BOX64ENV(rolling_log)) {
         printf_log(loglevel, "Last calls\n");
-        int j = (my_context->current_line+1)%cycle_log;
-        for (int i=0; i<cycle_log; ++i) {
-            int k = (i+j)%cycle_log;
+        int j = (my_context->current_line+1)%BOX64ENV(rolling_log);
+        for (int i=0; i<BOX64ENV(rolling_log); ++i) {
+            int k = (i+j)%BOX64ENV(rolling_log);
             if(my_context->log_call[k][0]) {
                 printf_log(loglevel, "%s => return %s\n", my_context->log_call[k], my_context->log_ret[k]);
             }
@@ -423,4 +425,4 @@ void x86Int3(x64emu_t* emu, uintptr_t* addr)
     printf_log(LOG_NONE, "Error: Calling 32bits wrapped function without box32 support built in\n");
     abort();
 }
-#endif
\ No newline at end of file
+#endif
