@@ -103,6 +103,9 @@
 
 #include "x64emu_private.h"
 #include "x64run_private.h"
+#include "env.h"
+
+extern box64env_t box64env;
 
 /*------------------------- Global Variables ------------------------------*/
 
@@ -488,7 +491,6 @@ uint8_t rcl8(x64emu_t *emu, uint8_t d, uint8_t s)
        3) B_(n-1) <- cf
        4) B_(n-2) .. B_0 <-  b_7 .. b_(8-(n-1))
 	 */
-	res = d;
 	if ((cnt = s % 9) != 0) {
         /* extract the new CARRY FLAG. */
         /* CF <-  b_(8-n)             */
@@ -518,10 +520,15 @@ uint8_t rcl8(x64emu_t *emu, uint8_t d, uint8_t s)
 		CONDITIONAL_SET_FLAG(cf, F_CF);
         /* OVERFLOW is set *IFF* cnt==1, then it is the 
            xor of CF and the most significant bit.  Blecck. */
-		if(cnt == 1)
-        	CONDITIONAL_SET_FLAG((cf ^ (res >> 7)) & 0x1, F_OF);
-
-    }
+		if(BOX64ENV(cputype))
+		   CONDITIONAL_SET_FLAG((cf ^ (res >> 7)) & 0x1, F_OF);
+	   else
+		   CONDITIONAL_SET_FLAG((XOR2(d >> 6)), F_OF);
+    } else if(s) {
+		res = d;
+		if(BOX64ENV(cputype))
+			CONDITIONAL_SET_FLAG((ACCESS_FLAG(F_CF) ^ (res >> 7)) & 0x1, F_OF);
+	}
 	return (uint8_t)res;
 }
 
@@ -531,7 +538,6 @@ uint16_t rcl16(x64emu_t *emu, uint16_t d, uint8_t s)
 	CHECK_FLAGS(emu);
 	s = s&0x1f;
 
-	res = d;
 	if ((cnt = s % 17) != 0) {
 		cf = (d >> (16 - cnt)) & 0x1;
 		res = (d << cnt) & 0xffff;
@@ -541,8 +547,14 @@ uint16_t rcl16(x64emu_t *emu, uint16_t d, uint8_t s)
 			res |= 1 << (cnt - 1);
 		}
 		CONDITIONAL_SET_FLAG(cf, F_CF);
-		if(cnt == 1)
-        	CONDITIONAL_SET_FLAG((cf ^ (res >> 15)) & 0x1, F_OF);
+		if(BOX64ENV(cputype))
+			CONDITIONAL_SET_FLAG((cf ^ (res >> 15)) & 0x1, F_OF);
+		else
+			CONDITIONAL_SET_FLAG((XOR2(d >> 14)), F_OF);
+	} else if(s) {
+		res = d;
+		if(BOX64ENV(cputype))
+			CONDITIONAL_SET_FLAG((ACCESS_FLAG(F_CF) ^ (res >> 15)) & 0x1, F_OF);
 	}
 	return (uint16_t)res;
 }
@@ -563,8 +575,10 @@ uint32_t rcl32(x64emu_t *emu, uint32_t d, uint8_t s)
 			res |= 1 << (cnt - 1);
 		}
 		CONDITIONAL_SET_FLAG(cf, F_CF);
-		if(cnt == 1)
-        	CONDITIONAL_SET_FLAG((cf ^ (res >> 31)) & 0x1, F_OF);
+		if(BOX64ENV(cputype))
+			CONDITIONAL_SET_FLAG((cf ^ (res >> 31)) & 0x1, F_OF);
+		else
+			CONDITIONAL_SET_FLAG((XOR2(d >> 30)), F_OF);
 	}
 	return res;
 }
@@ -585,8 +599,10 @@ uint64_t rcl64(x64emu_t *emu, uint64_t d, uint8_t s)
 			res |= 1LL << (cnt - 1);
 		}
 		CONDITIONAL_SET_FLAG(cf, F_CF);
-		if(cnt == 1)
-        	CONDITIONAL_SET_FLAG((cf ^ (res >> 63)) & 0x1, F_OF);
+		if(BOX64ENV(cputype))
+			CONDITIONAL_SET_FLAG((cf ^ (res >> 63)) & 0x1, F_OF);
+		else
+			CONDITIONAL_SET_FLAG((XOR2(d >> 62)), F_OF);
 	}
 	return res;
 }
@@ -627,7 +643,13 @@ uint8_t rcr8(x64emu_t *emu, uint8_t d, uint8_t s)
 	if ((cnt = s % 9) != 0) {
         /* extract the new CARRY FLAG. */
         /* CF <-  b_(n-1)              */
-        if (cnt == 1) {
+		ocf = ACCESS_FLAG(F_CF) != 0;
+		/* OVERFLOW is set *IFF* cnt==1, then it is the 
+		xor of CF and the most significant bit.  Blecck. */
+		/* parenthesized... */
+		if(!BOX64ENV(cputype))
+			CONDITIONAL_SET_FLAG((ocf ^ (d >> 7)) & 0x1, F_OF);
+		if (cnt == 1) {
             cf = d & 0x1;
             /* note hackery here.  Access_flag(..) evaluates to either
                0 if flag not set
@@ -636,12 +658,6 @@ uint8_t rcr8(x64emu_t *emu, uint8_t d, uint8_t s)
 			   0..1 in any representation of the flags register
                (i.e. packed bit array or unpacked.)
              */
-			ocf = ACCESS_FLAG(F_CF) != 0;
-			/* OVERFLOW is set *IFF* cnt==1, then it is the 
-			xor of CF and the most significant bit.  Blecck. */
-			/* parenthesized... */
-			CONDITIONAL_SET_FLAG((ocf ^ (d >> 7)) & 0x1,
-								 F_OF);
         } else
             cf = (d >> (cnt - 1)) & 0x1;
 
@@ -670,7 +686,10 @@ uint8_t rcr8(x64emu_t *emu, uint8_t d, uint8_t s)
         }
         /* set the new carry flag, based on the variable "cf" */
 		CONDITIONAL_SET_FLAG(cf, F_CF);
+
 	}
+	if(s && BOX64ENV(cputype))
+		CONDITIONAL_SET_FLAG((XOR2(res >> 6)), F_OF);
 	return (uint8_t)res;
 }
 
@@ -684,11 +703,11 @@ uint16_t rcr16(x64emu_t *emu, uint16_t d, uint8_t s)
 	/* rotate right through carry */
 	res = d;
 	if ((cnt = s % 17) != 0) {
+		ocf = ACCESS_FLAG(F_CF) != 0;
+		if(!BOX64ENV(cputype))
+			CONDITIONAL_SET_FLAG((ocf ^ (d >> 15)) & 0x1, F_OF);
 		if (cnt == 1) {
 			cf = d & 0x1;
-			ocf = ACCESS_FLAG(F_CF) != 0;
-			CONDITIONAL_SET_FLAG((ocf ^ (d >> 15)) & 0x1,
-								 F_OF);
 		} else
 			cf = (d >> (cnt - 1)) & 0x1;
 		mask = (1 << (16 - cnt)) - 1;
@@ -699,6 +718,8 @@ uint16_t rcr16(x64emu_t *emu, uint16_t d, uint8_t s)
 		}
 		CONDITIONAL_SET_FLAG(cf, F_CF);
 	}
+	if(s && BOX64ENV(cputype))
+		CONDITIONAL_SET_FLAG((XOR2(res >> 14)), F_OF);
 	return (uint16_t)res;
 }
 
@@ -712,13 +733,14 @@ uint32_t rcr32(x64emu_t *emu, uint32_t d, uint8_t s)
 	/* rotate right through carry */
 	res = d;
 	if ((cnt = s) != 0) {
+		ocf = ACCESS_FLAG(F_CF) != 0;
+		if(!BOX64ENV(cputype))
+			CONDITIONAL_SET_FLAG((ocf ^ (d >> 31)) & 0x1, F_OF);
 		if (cnt == 1) {
 			cf = d & 0x1;
-			ocf = ACCESS_FLAG(F_CF) != 0;
-			CONDITIONAL_SET_FLAG((ocf ^ (d >> 31)) & 0x1,
-								 F_OF);
-		} else
+		} else {
 			cf = (d >> (cnt - 1)) & 0x1;
+		}
 		mask = (1 << (32 - cnt)) - 1;
 		res = (d >> cnt) & mask;
 		if (cnt != 1)
@@ -728,6 +750,8 @@ uint32_t rcr32(x64emu_t *emu, uint32_t d, uint8_t s)
 		}
 		CONDITIONAL_SET_FLAG(cf, F_CF);
 	}
+	if(s && BOX64ENV(cputype))
+		CONDITIONAL_SET_FLAG((XOR2(res >> 30)), F_OF);
 	return res;
 }
 
@@ -741,13 +765,14 @@ uint64_t rcr64(x64emu_t *emu, uint64_t d, uint8_t s)
 	/* rotate right through carry */
 	res = d;
 	if ((cnt = s) != 0) {
+		ocf = ACCESS_FLAG(F_CF) != 0;
+		if(!BOX64ENV(cputype))
+			CONDITIONAL_SET_FLAG((ocf ^ (d >> 63)) & 0x1, F_OF);
 		if (cnt == 1) {
 			cf = d & 0x1;
-			ocf = ACCESS_FLAG(F_CF) != 0;
-			CONDITIONAL_SET_FLAG((ocf ^ (d >> 63)) & 0x1,
-								 F_OF);
-		} else
+		} else {
 			cf = (d >> (cnt - 1)) & 0x1;
+		}
 		mask = (1LL << (64 - cnt)) - 1;
 		res = (d >> cnt) & mask;
 		if (cnt != 1)
@@ -757,6 +782,8 @@ uint64_t rcr64(x64emu_t *emu, uint64_t d, uint8_t s)
 		}
 		CONDITIONAL_SET_FLAG(cf, F_CF);
 	}
+	if(s && BOX64ENV(cputype))
+		CONDITIONAL_SET_FLAG((XOR2(res >> 62)), F_OF);
 	return res;
 }
 /****************************************************************************
@@ -769,16 +796,16 @@ uint8_t rol8(x64emu_t *emu, uint8_t d, uint8_t s)
 
 	s = s&0x1f;
 	if(!s) return d;
+	CHECK_FLAGS(emu);
+	/* OF flag is set if s == 1; OF = CF _XOR_ MSB of result */
+	
+	if(!BOX64ENV(cputype)) CONDITIONAL_SET_FLAG(XOR2(d>>6), F_OF);
 
 	if((cnt = s % 8) != 0) {
 	d = (d << cnt) + ((d >> (8 - cnt)) & ((1 << cnt) - 1));
 	}
-	CHECK_FLAGS(emu);
 
-	/* OF flag is set if s == 1; OF = CF _XOR_ MSB of result */
-	if(s == 1) {
-	CONDITIONAL_SET_FLAG((d + (d >> 7)) & 1, F_OF);
-	}
+	if(BOX64ENV(cputype)) CONDITIONAL_SET_FLAG((d + (d >> 7)) & 1, F_OF);
 
 	/* set new CF; note that it is the LSB of the result */
 	CONDITIONAL_SET_FLAG(d & 0x1, F_CF);
@@ -792,18 +819,15 @@ uint16_t rol16(x64emu_t *emu, uint16_t d, uint8_t s)
 
 	s = s&0x1f;
 	if(!s) return d;
+	CHECK_FLAGS(emu);
 
+	if(!BOX64ENV(cputype)) CONDITIONAL_SET_FLAG(XOR2(d>>14), F_OF);
 	if((cnt = s % 16) != 0) {
 	d = (d << cnt) + ((d >> (16 - cnt)) & ((1 << cnt) - 1));
 	}
-	CHECK_FLAGS(emu);
 
-	/* OF flag is set if s == 1; OF = CF _XOR_ MSB of result */
-	if(s == 1) {
-	CONDITIONAL_SET_FLAG((d + (d >> 15)) & 1, F_OF);
-	}
+	if(BOX64ENV(cputype)) CONDITIONAL_SET_FLAG((d + (d >> 15)) & 1, F_OF);
 
-	/* set new CF; note that it is the LSB of the result */
 	CONDITIONAL_SET_FLAG(d & 0x1, F_CF);
 
 	return d;
@@ -815,18 +839,15 @@ uint32_t rol32(x64emu_t *emu, uint32_t d, uint8_t s)
 
 	s = s&0x1f;
 	if(!s) return d;
+	CHECK_FLAGS(emu);
 
+	if(!BOX64ENV(cputype)) CONDITIONAL_SET_FLAG(XOR2(d>>30), F_OF);
 	if((cnt = s % 32) != 0) {
 	d = (d << cnt) + ((d >> (32 - cnt)) & ((1 << cnt) - 1));
 	}
-	CHECK_FLAGS(emu);
 
-	/* OF flag is set if s == 1; OF = CF _XOR_ MSB of result */
-	if(s == 1) {
-	CONDITIONAL_SET_FLAG((d + (d >> 31)) & 1, F_OF);
-	}
+	if(BOX64ENV(cputype)) CONDITIONAL_SET_FLAG((d + (d >> 31)) & 1, F_OF);
 
-	/* set new CF; note that it is the LSB of the result */
 	CONDITIONAL_SET_FLAG(d & 0x1, F_CF);
 
 	return d;
@@ -838,18 +859,15 @@ uint64_t rol64(x64emu_t *emu, uint64_t d, uint8_t s)
 
 	s = s&0x3f;
 	if(!s) return d;
+	CHECK_FLAGS(emu);
 
+	if(!BOX64ENV(cputype)) CONDITIONAL_SET_FLAG(XOR2(d>>62), F_OF);
 	if((cnt = s % 64) != 0) {
 	d = (d << cnt) + ((d >> (64 - cnt)) & ((1L << cnt) - 1));
 	}
-	CHECK_FLAGS(emu);
 
-	/* OF flag is set if s == 1; OF = CF _XOR_ MSB of result */
-	if(s == 1) {
-	CONDITIONAL_SET_FLAG((d + (d >> 63)) & 1, F_OF);
-	}
+	if(BOX64ENV(cputype)) CONDITIONAL_SET_FLAG((d + (d >> 63)) & 1, F_OF);
 
-	/* set new CF; note that it is the LSB of the result */
 	CONDITIONAL_SET_FLAG(d & 0x1, F_CF);
 
 	return d;
@@ -865,17 +883,16 @@ uint8_t ror8(x64emu_t *emu, uint8_t d, uint8_t s)
 
 	s = s&0x1f;
 	if(!s) return d;
-
-	if((cnt = s % 8) != 0) {
-	d = (d << (8 - cnt)) + ((d >> (cnt)) & ((1 << (8 - cnt)) - 1));
-	}
 	CHECK_FLAGS(emu);
 
-	/* OF flag is set if s == 1; OF = MSB _XOR_ (M-1)SB of result */
-	if(s == 1) {
-	CONDITIONAL_SET_FLAG(XOR2(d >> 6), F_OF);
+	if(!BOX64ENV(cputype)) CONDITIONAL_SET_FLAG(((d >> 7)^d)&1, F_OF);
+
+	if((cnt = s % 8) != 0) {
+		d = (d << (8 - cnt)) + ((d >> (cnt)) & ((1 << (8 - cnt)) - 1));
 	}
 
+	if(BOX64ENV(cputype)) CONDITIONAL_SET_FLAG(XOR2(d >> 6), F_OF);
+
 	/* set new CF; note that it is the MSB of the result */
 	CONDITIONAL_SET_FLAG(d & (1 << 7), F_CF);
 
@@ -888,17 +905,16 @@ uint16_t ror16(x64emu_t *emu, uint16_t d, uint8_t s)
 
 	s = s&0x1f;
 	if(!s) return d;
-
-	if((cnt = s % 16) != 0) {
-	d = (d << (16 - cnt)) + ((d >> (cnt)) & ((1 << (16 - cnt)) - 1));
-	}
 	CHECK_FLAGS(emu);
 
-	/* OF flag is set if s == 1; OF = MSB _XOR_ (M-1)SB of result */
-	if(s == 1) {
-	CONDITIONAL_SET_FLAG(XOR2(d >> 14), F_OF);
+	if(!BOX64ENV(cputype)) CONDITIONAL_SET_FLAG(((d >> 15)^d)&1, F_OF);
+
+	if((cnt = s % 16) != 0) {
+		d = (d << (16 - cnt)) + ((d >> (cnt)) & ((1 << (16 - cnt)) - 1));
 	}
 
+	if(BOX64ENV(cputype)) CONDITIONAL_SET_FLAG(XOR2(d >> 14), F_OF);
+
 	/* set new CF; note that it is the MSB of the result */
 	CONDITIONAL_SET_FLAG(d & (1 << 15), F_CF);
 
@@ -911,17 +927,16 @@ uint32_t ror32(x64emu_t *emu, uint32_t d, uint8_t s)
 
 	s = s&0x1f;
 	if(!s) return d;
-
-	if((cnt = s % 32) != 0) {
-	d = (d << (32 - cnt)) + ((d >> (cnt)) & ((1 << (32 - cnt)) - 1));
-	}
 	CHECK_FLAGS(emu);
 
-	/* OF flag is set if s == 1; OF = MSB _XOR_ (M-1)SB of result */
-	if(s == 1) {
-	CONDITIONAL_SET_FLAG(XOR2(d >> 30), F_OF);
+	if(!BOX64ENV(cputype)) CONDITIONAL_SET_FLAG(((d >> 31)^d)&1, F_OF);
+
+	if((cnt = s % 32) != 0) {
+		d = (d << (32 - cnt)) + ((d >> (cnt)) & ((1 << (32 - cnt)) - 1));
 	}
 
+	if(BOX64ENV(cputype)) CONDITIONAL_SET_FLAG(XOR2(d >> 30), F_OF);
+
 	/* set new CF; note that it is the MSB of the result */
 	CONDITIONAL_SET_FLAG(d & (1 << 31), F_CF);
 
@@ -934,17 +949,16 @@ uint64_t ror64(x64emu_t *emu, uint64_t d, uint8_t s)
 
 	s = s&0x3f;
 	if(!s) return d;
-
-	if((cnt = s % 64) != 0) {
-	d = (d << (64 - cnt)) + ((d >> (cnt)) & ((1L << (64 - cnt)) - 1L));
-	}
 	CHECK_FLAGS(emu);
 
-	/* OF flag is set if s == 1; OF = MSB _XOR_ (M-1)SB of result */
-	if(s == 1) {
-	CONDITIONAL_SET_FLAG(XOR2(d >> 62), F_OF);
+	if(!BOX64ENV(cputype)) CONDITIONAL_SET_FLAG(((d >> 63)^d)&1, F_OF);
+
+	if((cnt = s % 64) != 0) {
+		d = (d << (64 - cnt)) + ((d >> (cnt)) & ((1L << (64 - cnt)) - 1L));
 	}
 
+	if(BOX64ENV(cputype)) CONDITIONAL_SET_FLAG(XOR2(d >> 62), F_OF);
+
 	/* set new CF; note that it is the MSB of the result */
 	CONDITIONAL_SET_FLAG(d & (1L << 63), F_CF);
 
@@ -964,33 +978,39 @@ uint16_t shld16 (x64emu_t *emu, uint16_t d, uint16_t fill, uint8_t s)
 		return d;
 	RESET_FLAGS(emu);
 	if (s < 16) {
-		if (cnt > 0) {
-			res = (d << cnt) | (fill >> (16-cnt));
-			cf = d & (1 << (16 - cnt));
-			CONDITIONAL_SET_FLAG(cf, F_CF);
-			CONDITIONAL_SET_FLAG((res & 0xffff) == 0, F_ZF);
-			CONDITIONAL_SET_FLAG(res & 0x8000, F_SF);
-			CONDITIONAL_SET_FLAG(PARITY(res & 0xff), F_PF);
-		} else {
-			res = d;
-		}
-		if (cnt == 1) {
-			CONDITIONAL_SET_FLAG(((res ^ d) >> 15)&1, F_OF);
-		} else {
-			CONDITIONAL_SET_FLAG((d >> 15)&1, F_OF);
-		}
+		res = (d << cnt) | (fill >> (16-cnt));
+		cf = d & (1 << (16 - cnt));
+		CONDITIONAL_SET_FLAG(cf, F_CF);
+		CONDITIONAL_SET_FLAG((res & 0xffff) == 0, F_ZF);
+		CONDITIONAL_SET_FLAG(res & 0x8000, F_SF);
+		CONDITIONAL_SET_FLAG(PARITY(res & 0xff), F_PF);
 	} else {
 		res = (fill << (cnt)) | (d >> (16 - cnt));
 		if(s==16)
 			cf = d & 1;
 		else
 			cf = fill & (1 << (16 - cnt));
-		CONDITIONAL_SET_FLAG(cf, F_CF);
+		if(BOX64ENV(cputype) && (s>16)) {
+			CLEAR_FLAG(F_CF);
+		} else {
+			CONDITIONAL_SET_FLAG(cf, F_CF);
+		}
 		CONDITIONAL_SET_FLAG((res & 0xffff) == 0, F_ZF);
 		CONDITIONAL_SET_FLAG(res & 0x8000, F_SF);
 		CONDITIONAL_SET_FLAG(PARITY(res & 0xff), F_PF);
-		CLEAR_FLAG(F_OF);
 	}
+	if (BOX64ENV(cputype)) {
+		if(s>15)
+			CONDITIONAL_SET_FLAG(ACCESS_FLAG(F_CF), F_OF);
+		else
+			CONDITIONAL_SET_FLAG((ACCESS_FLAG(F_CF) ^ (res>>15))&1, F_OF);
+	} else {
+		CONDITIONAL_SET_FLAG(XOR2(d>>14), F_OF);
+	}
+	if (BOX64ENV(cputype))
+		SET_FLAG(F_AF);
+	else
+		CLEAR_FLAG(F_AF);
 	return (uint16_t)res;
 }
 
@@ -1013,11 +1033,15 @@ uint32_t shld32 (x64emu_t *emu, uint32_t d, uint32_t fill, uint8_t s)
 	} else {
 		res = d;
 	}
-	if (cnt == 1) {
-		CONDITIONAL_SET_FLAG(((res ^ d) >> 31)&1, F_OF);
+	if (BOX64ENV(cputype)) {
+		CONDITIONAL_SET_FLAG((ACCESS_FLAG(F_CF) ^ (res>>31))&1, F_OF);
 	} else {
-		CONDITIONAL_SET_FLAG((d >> 31)&1, F_OF);
+		CONDITIONAL_SET_FLAG(XOR2(d>>30), F_OF);
 	}
+	if (BOX64ENV(cputype))
+		SET_FLAG(F_AF);
+	else
+		CLEAR_FLAG(F_AF);
 	return res;
 }
 
@@ -1040,11 +1064,15 @@ uint64_t shld64 (x64emu_t *emu, uint64_t d, uint64_t fill, uint8_t s)
 	} else {
 		res = d;
 	}
-	if (cnt == 1) {
-		CONDITIONAL_SET_FLAG(((res ^ d) >> 63)&1, F_OF);
+	if (BOX64ENV(cputype)) {
+		CONDITIONAL_SET_FLAG((ACCESS_FLAG(F_CF) ^ (res>>63))&1, F_OF);
 	} else {
-		CONDITIONAL_SET_FLAG((d >> 63)&1, F_OF);
+		CONDITIONAL_SET_FLAG(XOR2(d>>62), F_OF);
 	}
+	if (BOX64ENV(cputype))
+		SET_FLAG(F_AF);
+	else
+		CLEAR_FLAG(F_AF);
 	return res;
 }
 
@@ -1073,22 +1101,20 @@ uint16_t shrd16 (x64emu_t *emu, uint16_t d, uint16_t fill, uint8_t s)
 			res = d;
 		}
 
-		if (cnt == 1) {
-			CONDITIONAL_SET_FLAG(((res ^ d) >> 15)&1, F_OF);
-        } else {
-			CONDITIONAL_SET_FLAG((d >> 15)&1, F_OF);
-        }
 	} else {
 		if(s==16)
 			cf = d & (1 << 15);
 		else
 			cf = fill & (1 << (cnt - 1));
 		res = (fill >> cnt) | (d << (16 - cnt));
-		CONDITIONAL_SET_FLAG(cf, F_CF);
+		if(BOX64ENV(cputype) && (s>16)) {
+			CLEAR_FLAG(F_CF);
+		} else {
+			CONDITIONAL_SET_FLAG(cf, F_CF);
+		}
 		CONDITIONAL_SET_FLAG((res & 0xffff) == 0, F_ZF);
 		CONDITIONAL_SET_FLAG(res & 0x8000, F_SF);
 		CONDITIONAL_SET_FLAG(PARITY(res & 0xff), F_PF);
-		CLEAR_FLAG(F_OF);
 	#if 0
 		res = 0;
 		CLEAR_FLAG(F_CF);
@@ -1098,6 +1124,15 @@ uint16_t shrd16 (x64emu_t *emu, uint16_t d, uint16_t fill, uint8_t s)
 		CLEAR_FLAG(F_PF);
 	#endif
     }
+	if (BOX64ENV(cputype)) {
+		CONDITIONAL_SET_FLAG(XOR2(res>>14), F_OF);
+	} else {
+		CONDITIONAL_SET_FLAG((fill^(d>>15))&1, F_OF);
+	}
+	if (BOX64ENV(cputype))
+		SET_FLAG(F_AF);
+	else
+		CLEAR_FLAG(F_AF);
 	return (uint16_t)res;
 }
 
@@ -1120,11 +1155,15 @@ uint32_t shrd32 (x64emu_t *emu, uint32_t d, uint32_t fill, uint8_t s)
 	} else {
 		res = d;
 	}
-	if (cnt == 1) {
-		CONDITIONAL_SET_FLAG(((res ^ d) >> 31)&1, F_OF);
+	if (BOX64ENV(cputype)) {
+		CONDITIONAL_SET_FLAG(XOR2(res>>30), F_OF);
 	} else {
-		CONDITIONAL_SET_FLAG((d >> 31)&1, F_OF);
+		CONDITIONAL_SET_FLAG((fill^(d>>31))&1, F_OF);
 	}
+	if (BOX64ENV(cputype))
+		SET_FLAG(F_AF);
+	else
+		CLEAR_FLAG(F_AF);
 	return res;
 }
 
@@ -1148,11 +1187,15 @@ uint64_t shrd64 (x64emu_t *emu, uint64_t d, uint64_t fill, uint8_t s)
 	} else {
 		res = d;
 	}
-	if (cnt == 1) {
-		CONDITIONAL_SET_FLAG(((res ^ d) >> 63)&1, F_OF);
+	if (BOX64ENV(cputype)) {
+		CONDITIONAL_SET_FLAG(XOR2(res>>62), F_OF);
 	} else {
-		CONDITIONAL_SET_FLAG((d >> 63)&1, F_OF);
+		CONDITIONAL_SET_FLAG((fill^(d>>63))&1, F_OF);
 	}
+	if (BOX64ENV(cputype))
+		SET_FLAG(F_AF);
+	else
+		CLEAR_FLAG(F_AF);
 	return res;
 }
 /****************************************************************************
@@ -1315,18 +1358,16 @@ void test64(x64emu_t *emu, uint64_t d, uint64_t s)
 REMARKS:
 Implements the IDIV instruction and side effects.
 ****************************************************************************/
-extern int box64_dynarec_test;
+
 void idiv8(x64emu_t *emu, uint8_t s)
 {
     int32_t dvd, quot, mod;
-	RESET_FLAGS(emu);
-	if(box64_dynarec_test) {
-		CLEAR_FLAG(F_CF);
-		CLEAR_FLAG(F_AF);
+	if(BOX64ENV(cputype)) {
+		CHECK_FLAGS(emu);
+		SET_FLAG(F_AF);
 		CLEAR_FLAG(F_PF);
 		CLEAR_FLAG(F_ZF);
 		CLEAR_FLAG(F_SF);
-		CLEAR_FLAG(F_OF);
 	}
 
 	dvd = (int16_t)R_AX;
@@ -1348,14 +1389,12 @@ void idiv8(x64emu_t *emu, uint8_t s)
 void idiv16(x64emu_t *emu, uint16_t s)
 {
 	int32_t dvd, quot, mod;
-	RESET_FLAGS(emu);
-	if(box64_dynarec_test) {
-		CLEAR_FLAG(F_CF);
-		CLEAR_FLAG(F_AF);
+	if(BOX64ENV(cputype)) {
+		CHECK_FLAGS(emu);
+		SET_FLAG(F_AF);
 		CLEAR_FLAG(F_PF);
 		CLEAR_FLAG(F_ZF);
 		CLEAR_FLAG(F_SF);
-		CLEAR_FLAG(F_OF);
 	}
 
 	dvd = (((int32_t)R_DX) << 16) | R_AX;
@@ -1378,14 +1417,12 @@ void idiv16(x64emu_t *emu, uint16_t s)
 void idiv32(x64emu_t *emu, uint32_t s)
 {
 	int64_t dvd, quot, mod;
-	RESET_FLAGS(emu);
-	if(box64_dynarec_test) {
-		CLEAR_FLAG(F_CF);
-		CLEAR_FLAG(F_AF);
+	if(BOX64ENV(cputype)) {
+		CHECK_FLAGS(emu);
+		SET_FLAG(F_AF);
 		CLEAR_FLAG(F_PF);
 		CLEAR_FLAG(F_ZF);
 		CLEAR_FLAG(F_SF);
-		CLEAR_FLAG(F_OF);
 	}
 
 	dvd = (((int64_t)R_EDX) << 32) | R_EAX;
@@ -1408,14 +1445,12 @@ void idiv32(x64emu_t *emu, uint32_t s)
 void idiv64(x64emu_t *emu, uint64_t s)
 {
 	__int128 dvd, quot, mod;
-	RESET_FLAGS(emu);
-	if(box64_dynarec_test) {
-		CLEAR_FLAG(F_CF);
-		CLEAR_FLAG(F_AF);
+	if(BOX64ENV(cputype)) {
+		CHECK_FLAGS(emu);
+		SET_FLAG(F_AF);
 		CLEAR_FLAG(F_PF);
 		CLEAR_FLAG(F_ZF);
 		CLEAR_FLAG(F_SF);
-		CLEAR_FLAG(F_OF);
 	}
 
 	dvd = (((__int128)R_RDX) << 64) | R_RAX;
@@ -1442,13 +1477,17 @@ void div8(x64emu_t *emu, uint8_t s)
 {
 	uint32_t dvd, div, mod;
 	RESET_FLAGS(emu);
-	if(box64_dynarec_test) {
-		CLEAR_FLAG(F_CF);
-		CLEAR_FLAG(F_AF);
-		CLEAR_FLAG(F_PF);
+	CLEAR_FLAG(F_CF);
+	CLEAR_FLAG(F_SF);
+	CLEAR_FLAG(F_OF);
+	if(BOX64ENV(cputype)) {
+		SET_FLAG(F_AF);
 		CLEAR_FLAG(F_ZF);
-		CLEAR_FLAG(F_SF);
-		CLEAR_FLAG(F_OF);
+		CLEAR_FLAG(F_PF);
+	} else {
+		CLEAR_FLAG(F_AF);
+		SET_FLAG(F_ZF);
+		SET_FLAG(F_PF);
 	}
 
 	dvd = R_AX;
@@ -1470,13 +1509,17 @@ void div16(x64emu_t *emu, uint16_t s)
 {
 	uint32_t dvd, div, mod;
 	RESET_FLAGS(emu);
-	if(box64_dynarec_test) {
-		CLEAR_FLAG(F_CF);
-		CLEAR_FLAG(F_AF);
-		CLEAR_FLAG(F_PF);
+	CLEAR_FLAG(F_CF);
+	CLEAR_FLAG(F_SF);
+	CLEAR_FLAG(F_OF);
+	if(BOX64ENV(cputype)) {
+		SET_FLAG(F_AF);
 		CLEAR_FLAG(F_ZF);
-		CLEAR_FLAG(F_SF);
-		CLEAR_FLAG(F_OF);
+		CLEAR_FLAG(F_PF);
+	} else {
+		CLEAR_FLAG(F_AF);
+		SET_FLAG(F_ZF);
+		SET_FLAG(F_PF);
 	}
 
 	dvd = (((uint32_t)R_DX) << 16) | R_AX;
@@ -1499,13 +1542,17 @@ void div32(x64emu_t *emu, uint32_t s)
 {
 	uint64_t dvd, div, mod;
 	RESET_FLAGS(emu);
-	if(box64_dynarec_test) {
-		CLEAR_FLAG(F_CF);
-		CLEAR_FLAG(F_AF);
-		CLEAR_FLAG(F_PF);
+	CLEAR_FLAG(F_CF);
+	CLEAR_FLAG(F_SF);
+	CLEAR_FLAG(F_OF);
+	if(BOX64ENV(cputype)) {
+		SET_FLAG(F_AF);
 		CLEAR_FLAG(F_ZF);
-		CLEAR_FLAG(F_SF);
-		CLEAR_FLAG(F_OF);
+		CLEAR_FLAG(F_PF);
+	} else {
+		CLEAR_FLAG(F_AF);
+		SET_FLAG(F_ZF);
+		SET_FLAG(F_PF);
 	}
 
 	dvd = (((uint64_t)R_EDX) << 32) | R_EAX;
@@ -1528,13 +1575,17 @@ void div64(x64emu_t *emu, uint64_t s)
 {
 	__int128 dvd, div, mod;
 	RESET_FLAGS(emu);
-	if(box64_dynarec_test) {
-		CLEAR_FLAG(F_CF);
-		CLEAR_FLAG(F_AF);
-		CLEAR_FLAG(F_PF);
+	CLEAR_FLAG(F_CF);
+	CLEAR_FLAG(F_SF);
+	CLEAR_FLAG(F_OF);
+	if(BOX64ENV(cputype)) {
+		SET_FLAG(F_AF);
 		CLEAR_FLAG(F_ZF);
-		CLEAR_FLAG(F_SF);
-		CLEAR_FLAG(F_OF);
+		CLEAR_FLAG(F_PF);
+	} else {
+		CLEAR_FLAG(F_AF);
+		SET_FLAG(F_ZF);
+		SET_FLAG(F_PF);
 	}
 
 	dvd = (((__int128)R_RDX) << 64) | R_RAX;
