@@ -178,6 +178,13 @@ uintptr_t Run64(x64emu_t *emu, rex_t rex, int seg, uintptr_t addr)
                             return 0;
                     }
                     break;
+
+                case 0x18:
+                case 0x19:
+                case 0x1F: /* NOP (multi-byte) */
+                    nextop = F8;
+                    FAKEED(0);
+                    break;
                 case 0x28:
                     switch(rep) {
                         case 0: /* MOVAPS Gx, FS:Ex */
@@ -321,6 +328,12 @@ uintptr_t Run64(x64emu_t *emu, rex_t rex, int seg, uintptr_t addr)
                                 else
                                     CLEAR_FLAG(F_CF);
                             }
+                            if (BOX64ENV(dynarec_test)) {
+                                CLEAR_FLAG(F_OF);
+                                CLEAR_FLAG(F_SF);
+                                CLEAR_FLAG(F_AF);
+                                CLEAR_FLAG(F_PF);
+                            }
                             break;
                         case 5:             /* BTS Ed, Ib */
                             CHECK_FLAGS(emu);
@@ -345,6 +358,12 @@ uintptr_t Run64(x64emu_t *emu, rex_t rex, int seg, uintptr_t addr)
                                 if(MODREG)
                                     ED->dword[1] = 0;
                             }
+                            if (BOX64ENV(dynarec_test)) {
+                                CLEAR_FLAG(F_OF);
+                                CLEAR_FLAG(F_SF);
+                                CLEAR_FLAG(F_AF);
+                                CLEAR_FLAG(F_PF);
+                            }
                             break;
                         case 6:             /* BTR Ed, Ib */
                             CHECK_FLAGS(emu);
@@ -367,6 +386,12 @@ uintptr_t Run64(x64emu_t *emu, rex_t rex, int seg, uintptr_t addr)
                                 if(MODREG)
                                     ED->dword[1] = 0;
                             }
+                            if (BOX64ENV(dynarec_test)) {
+                                CLEAR_FLAG(F_OF);
+                                CLEAR_FLAG(F_SF);
+                                CLEAR_FLAG(F_AF);
+                                CLEAR_FLAG(F_PF);
+                            }
                             break;
                         case 7:             /* BTC Ed, Ib */
                             CHECK_FLAGS(emu);
@@ -389,6 +414,12 @@ uintptr_t Run64(x64emu_t *emu, rex_t rex, int seg, uintptr_t addr)
                                 if(MODREG)
                                     ED->dword[1] = 0;
                             }
+                            if (BOX64ENV(dynarec_test)) {
+                                CLEAR_FLAG(F_OF);
+                                CLEAR_FLAG(F_SF);
+                                CLEAR_FLAG(F_AF);
+                                CLEAR_FLAG(F_PF);
+                            }
                             break;
 
                         default:
