@@ -19,30 +19,34 @@
 #include "x64trace.h"
 #include "x87emu_private.h"
 #include "box64context.h"
+#include "signals.h"
 #include "bridge.h"
+#ifdef DYNAREC
+#include "custommem.h"
+#endif
 
 #include "modrm.h"
 #include "x64compstrings.h"
 
 static uint8_t ff_mult(uint8_t a, uint8_t b)
 {
-	int retval = 0;
+    int retval = 0;
 
-	for(int i = 0; i < 8; i++) {
-		if((b & 1) == 1)
-			retval ^= a;
+    for(int i = 0; i < 8; i++) {
+        if((b & 1) == 1)
+            retval ^= a;
 
-		if((a & 0x80)) {
-			a <<= 1;
-			a  ^= 0x1b;
-		} else {
-			a <<= 1;
-		}
+        if((a & 0x80)) {
+            a <<= 1;
+            a  ^= 0x1b;
+        } else {
+            a <<= 1;
+        }
 
-		b >>= 1;
-	}
+        b >>= 1;
+    }
 
-	return retval;
+    return retval;
 }
 
 #ifdef TEST_INTERPRETER
@@ -59,7 +63,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
     uint16_t tmp16u;
     int32_t tmp32s;
     uint32_t tmp32u;
-    uint64_t tmp64u;
+    uint64_t tmp64u, tmp64u2;
     int64_t tmp64s, i64[4];
     float tmpf;
     double tmpd;
@@ -67,7 +71,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
     int is_nan;
     #endif
     reg64_t *oped, *opgd;
-    sse_regs_t *opex, *opgx, eax1, *opex2;
+    sse_regs_t *opex, *opgx, eax1, *opex2, eax2;
     mmx87_regs_t *opem, *opgm;
     // AES opcodes constants
                             //   A0 B1 C2 D3 E4 F5 G6 H7 I8 J9 Ka Lb Mc Nd Oe Pf
@@ -170,6 +174,8 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         ED->q[0] = GX->q[1];
         break;
 
+    case 0x18:
+    case 0x19:
     case 0x1F:                      /* NOP (multi-byte) */
         nextop = F8;
         FAKEED(0);
@@ -332,6 +338,10 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 nextop = F8;
                 GETEX(0);
                 GETGX;
+                if(GX==EX) {
+                    eax1 = *EX;
+                    EX=&eax1;
+                }
                 for (int i=0; i<8; ++i) {
                     tmp32s = (int32_t)(GX->ub[i*2+0])*EX->sb[i*2+0] + (int32_t)(GX->ub[i*2+1])*EX->sb[i*2+1];
                     GX->sw[i] = (tmp32s>32767)?32767:((tmp32s<-32768)?-32768:tmp32s);
@@ -694,6 +704,15 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 GX->ud[1] = 0;
                 break;
 
+            case 0x82:  /* INVPCID */
+                nextop = F8;
+                GETED(0);
+                // this is a privilege opcode...
+                #ifndef TEST_INTERPRETER
+                emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
+                #endif
+                break;
+
             case 0xDB:  /* AESIMC Gx, Ex */
                 nextop = F8;
                 GETEX(0);
@@ -731,14 +750,14 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                     eax1.ub[i] = subbytes[eax1.ub[i]];
                 //STATE ← MixColumns( STATE );
                 for(int j=0; j<4; ++j) {
-                    GX->ub[0+j*4] = ff_mult(0x02, eax1.ub[0+j*4]) ^ ff_mult(0x03, eax1.ub[1+j*4]) ^               eax1.ub[2+j*4]  ^               eax1.ub[3+j*4] ;
-                    GX->ub[1+j*4] =               eax1.ub[0+j*4]  ^ ff_mult(0x02, eax1.ub[1+j*4]) ^ ff_mult(0x03, eax1.ub[2+j*4]) ^               eax1.ub[3+j*4] ;
-                    GX->ub[2+j*4] =               eax1.ub[0+j*4]  ^               eax1.ub[1+j*4]  ^ ff_mult(0x02, eax1.ub[2+j*4]) ^ ff_mult(0x03, eax1.ub[3+j*4]);
-                    GX->ub[3+j*4] = ff_mult(0x03, eax1.ub[0+j*4]) ^               eax1.ub[1+j*4]  ^               eax1.ub[2+j*4]  ^ ff_mult(0x02, eax1.ub[3+j*4]);
+                    eax2.ub[0+j*4] = ff_mult(0x02, eax1.ub[0+j*4]) ^ ff_mult(0x03, eax1.ub[1+j*4]) ^               eax1.ub[2+j*4]  ^               eax1.ub[3+j*4] ;
+                    eax2.ub[1+j*4] =               eax1.ub[0+j*4]  ^ ff_mult(0x02, eax1.ub[1+j*4]) ^ ff_mult(0x03, eax1.ub[2+j*4]) ^               eax1.ub[3+j*4] ;
+                    eax2.ub[2+j*4] =               eax1.ub[0+j*4]  ^               eax1.ub[1+j*4]  ^ ff_mult(0x02, eax1.ub[2+j*4]) ^ ff_mult(0x03, eax1.ub[3+j*4]);
+                    eax2.ub[3+j*4] = ff_mult(0x03, eax1.ub[0+j*4]) ^               eax1.ub[1+j*4]  ^               eax1.ub[2+j*4]  ^ ff_mult(0x02, eax1.ub[3+j*4]);
                 }
                 //DEST[127:0] ← STATE XOR RoundKey;
-                GX->q[0] ^= EX->q[0];
-                GX->q[1] ^= EX->q[1];
+                GX->q[0] = eax2.q[0] ^ EX->q[0];
+                GX->q[1] = eax2.q[1] ^ EX->q[1];
                 break;
             case 0xDD:  /* AESENCLAST Gx, Ex */
                 nextop = F8;
@@ -751,10 +770,10 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                     eax1.ub[i] = GX->ub[shiftrows[i]];
                 //STATE ← SubBytes( STATE );
                 for(int i=0; i<16; ++i)
-                    GX->ub[i] = subbytes[eax1.ub[i]];
+                    eax1.ub[i] = subbytes[eax1.ub[i]];
                 //DEST[127:0] ← STATE XOR RoundKey;
-                GX->q[0] ^= EX->q[0];
-                GX->q[1] ^= EX->q[1];
+                GX->q[0] = eax1.q[0] ^ EX->q[0];
+                GX->q[1] = eax1.q[1] ^ EX->q[1];
                 break;
             case 0xDE:  /* AESDEC Gx, Ex */
                 nextop = F8;
@@ -770,14 +789,14 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                     eax1.ub[i] = invsubbytes[eax1.ub[i]];
                 //STATE ← InvMixColumns( STATE );
                 for(int j=0; j<4; ++j) {
-                    GX->ub[0+j*4] = ff_mult(0x0E, eax1.ub[0+j*4]) ^ ff_mult(0x0B, eax1.ub[1+j*4]) ^ ff_mult(0x0D, eax1.ub[2+j*4]) ^ ff_mult(0x09, eax1.ub[3+j*4]);
-                    GX->ub[1+j*4] = ff_mult(0x09, eax1.ub[0+j*4]) ^ ff_mult(0x0E, eax1.ub[1+j*4]) ^ ff_mult(0x0B, eax1.ub[2+j*4]) ^ ff_mult(0x0D, eax1.ub[3+j*4]);
-                    GX->ub[2+j*4] = ff_mult(0x0D, eax1.ub[0+j*4]) ^ ff_mult(0x09, eax1.ub[1+j*4]) ^ ff_mult(0x0E, eax1.ub[2+j*4]) ^ ff_mult(0x0B, eax1.ub[3+j*4]);
-                    GX->ub[3+j*4] = ff_mult(0x0B, eax1.ub[0+j*4]) ^ ff_mult(0x0D, eax1.ub[1+j*4]) ^ ff_mult(0x09, eax1.ub[2+j*4]) ^ ff_mult(0x0E, eax1.ub[3+j*4]);
+                    eax2.ub[0+j*4] = ff_mult(0x0E, eax1.ub[0+j*4]) ^ ff_mult(0x0B, eax1.ub[1+j*4]) ^ ff_mult(0x0D, eax1.ub[2+j*4]) ^ ff_mult(0x09, eax1.ub[3+j*4]);
+                    eax2.ub[1+j*4] = ff_mult(0x09, eax1.ub[0+j*4]) ^ ff_mult(0x0E, eax1.ub[1+j*4]) ^ ff_mult(0x0B, eax1.ub[2+j*4]) ^ ff_mult(0x0D, eax1.ub[3+j*4]);
+                    eax2.ub[2+j*4] = ff_mult(0x0D, eax1.ub[0+j*4]) ^ ff_mult(0x09, eax1.ub[1+j*4]) ^ ff_mult(0x0E, eax1.ub[2+j*4]) ^ ff_mult(0x0B, eax1.ub[3+j*4]);
+                    eax2.ub[3+j*4] = ff_mult(0x0B, eax1.ub[0+j*4]) ^ ff_mult(0x0D, eax1.ub[1+j*4]) ^ ff_mult(0x09, eax1.ub[2+j*4]) ^ ff_mult(0x0E, eax1.ub[3+j*4]);
                 }
                 //DEST[127:0] ← STATE XOR RoundKey;
-                GX->q[0] ^= EX->q[0];
-                GX->q[1] ^= EX->q[1];
+                GX->q[0] = eax2.q[0] ^ EX->q[0];
+                GX->q[1] = eax2.q[1] ^ EX->q[1];
                 break;
             case 0xDF:  /* AESDECLAST Gx, Ex */
                 nextop = F8;
@@ -790,10 +809,10 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                     eax1.ub[i] = GX->ub[invshiftrows[i]];
                 //STATE ← InvSubBytes( STATE );
                 for(int i=0; i<16; ++i)
-                    GX->ub[i] = invsubbytes[eax1.ub[i]];
+                    eax1.ub[i] = invsubbytes[eax1.ub[i]];
                 //DEST[127:0] ← STATE XOR RoundKey;
-                GX->q[0] ^= EX->q[0];
-                GX->q[1] ^= EX->q[1];
+                GX->q[0] = eax1.q[0] ^ EX->q[0];
+                GX->q[1] = eax1.q[1] ^ EX->q[1];
                 break;
             case 0xF0: /* MOVBE Gw, Ew */
                 nextop = F8;
@@ -807,6 +826,33 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 GETGD;
                 ED->word[0] = __builtin_bswap16(GD->word[0]);
                 break;
+
+            case 0xF6: /* ADCX Gd, Rd */
+                nextop = F8;
+                GETED(0);
+                GETGD;
+                CHECK_FLAGS(emu);
+                if(rex.w) {
+                    if (ACCESS_FLAG(F_CF)) {
+                        tmp64u = 1 + (GD->q[0] & 0xFFFFFFFF) + (ED->q[0] & 0xFFFFFFFF);
+                        tmp64u2 = 1 + GD->q[0] + ED->q[0];
+                        }
+                    else {
+                        tmp64u = (GD->q[0] & 0xFFFFFFFF) + (ED->q[0] & 0xFFFFFFFF);
+                        tmp64u2 = GD->q[0] + ED->q[0];
+                        }
+                    tmp64u = (tmp64u >> 32) + (GD->q[0] >> 32) + (ED->q[0] >> 32);
+                    CONDITIONAL_SET_FLAG(tmp64u & 0x100000000LL, F_CF);
+                    GD->q[0] = tmp64u2;
+                } else {
+                    if (ACCESS_FLAG(F_CF))
+                        GD->q[0] = 1LL + GD->dword[0] + ED->dword[0];
+                    else
+                        GD->q[0] = (uint64_t)GD->dword[0] + ED->dword[0];
+                    CONDITIONAL_SET_FLAG(GD->q[0] & 0x100000000LL, F_CF);
+                    GD->dword[1] = 0;
+                }
+                break;
             default:
                 return 0;
         }
@@ -936,7 +982,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 }
                 break;
 
-            case 0x0C:  /* PBLENDPS Gx, Ex, Ib */
+            case 0x0C: /* BLENDPS Gx, Ex, Ib */
                 nextop = F8;
                 GETEX(1);
                 GETGX;
@@ -946,7 +992,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                         GX->ud[i] = EX->ud[i];
                 }
                 break;
-            case 0x0D:  /* PBLENDPD Gx, Ex, Ib */
+            case 0x0D: /* BLENDPD Gx, Ex, Ib */
                 nextop = F8;
                 GETEX(1);
                 GETGX;
@@ -1083,6 +1129,10 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 nextop = F8;
                 GETEX(1);
                 GETGX;
+                if(GX==EX) {
+                    eax1 = *EX;
+                    EX=&eax1;
+                }
                 tmp8u = F8;
                 {
                     int src = tmp8u&3;
@@ -1127,12 +1177,12 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 tmp32u = sse42_compare_string_explicit_len(emu, EX, R_EDX, GX, R_EAX, tmp8u);
                 if(tmp8u&0b1000000) {
                     switch(tmp8u&1) {
-                        case 0: for(int i=0; i<16; ++i) GX->ub[i] = ((tmp32u>>i)&1)?0xff:0x00; break;
-                        case 1: for(int i=0; i<8; ++i) GX->uw[i] = ((tmp32u>>i)&1)?0xffff:0x0000; break;
+                        case 0: for(int i=0; i<16; ++i) emu->xmm[0].ub[i] = ((tmp32u>>i)&1)?0xff:0x00; break;
+                        case 1: for(int i=0; i<8; ++i) emu->xmm[0].uw[i] = ((tmp32u>>i)&1)?0xffff:0x0000; break;
                     }
                 } else {
-                    GX->q[1] = GX->q[0] = 0;
-                    GX->uw[0] = tmp32u;
+                    emu->xmm[0].q[1] = emu->xmm[0].q[0] = 0;
+                    emu->xmm[0].uw[0] = tmp32u;
                 }
                 break;
             case 0x61:  /* PCMPESTRI */
@@ -1156,12 +1206,12 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 tmp32u = sse42_compare_string_implicit_len(emu, EX, GX, tmp8u);
                 if(tmp8u&0b1000000) {
                     switch(tmp8u&1) {
-                        case 0: for(int i=0; i<16; ++i) GX->ub[i] = ((tmp32u>>i)&1)?0xff:0x00; break;
-                        case 1: for(int i=0; i<8; ++i) GX->uw[i] = ((tmp32u>>i)&1)?0xffff:0x0000; break;
+                        case 0: for(int i=0; i<16; ++i) emu->xmm[0].ub[i] = ((tmp32u>>i)&1)?0xff:0x00; break;
+                        case 1: for(int i=0; i<8; ++i) emu->xmm[0].uw[i] = ((tmp32u>>i)&1)?0xffff:0x0000; break;
                     }
                 } else {
-                    GX->q[1] = GX->q[0] = 0;
-                    GX->uw[0] = tmp32u;
+                    emu->xmm[0].q[1] = emu->xmm[0].q[0] = 0;
+                    emu->xmm[0].uw[0] = tmp32u;
                 }
                 break;
             case 0x63:  /* PCMPISTRI */
@@ -1379,32 +1429,23 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         if (isnan(GX->d[1]) || isnan(EX->d[1]) || isgreater(EX->d[1], GX->d[1]))
             GX->d[1] = EX->d[1];
         break;
-
     case 0x60:  /* PUNPCKLBW Gx,Ex */
         nextop = F8;
         GETEX(0);
         GETGX;
-        for(int i=7; i>0; --i)  // 0 is untouched
+        for(int i=7; i>=0; --i) {
             GX->ub[2 * i] = GX->ub[i];
-        if(GX==EX)
-            for(int i=0; i<8; ++i)
-                GX->ub[2 * i + 1] = GX->ub[2 * i];
-        else
-            for(int i=0; i<8; ++i)
-                GX->ub[2 * i + 1] = EX->ub[i];
+            GX->ub[2 * i + 1] = EX->ub[i];
+        }
         break;
     case 0x61:  /* PUNPCKLWD Gx,Ex */
         nextop = F8;
         GETEX(0);
         GETGX;
-        for(int i=3; i>0; --i)
+        for(int i=3; i>=0; --i) {
             GX->uw[2 * i] = GX->uw[i];
-        if(GX==EX)
-            for(int i=0; i<4; ++i)
-                GX->uw[2 * i + 1] = GX->uw[2 * i];
-        else
-            for(int i=0; i<4; ++i)
-                GX->uw[2 * i + 1] = EX->uw[i];
+            GX->uw[2 * i + 1] = EX->uw[i];
+        }
         break;
     case 0x62:  /* PUNPCKLDQ Gx,Ex */
         nextop = F8;
@@ -1469,27 +1510,19 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         nextop = F8;
         GETEX(0);
         GETGX;
-        for(int i=0; i<8; ++i)
+        for(int i=0; i<8; ++i) {
             GX->ub[2 * i] = GX->ub[i + 8];
-        if(GX==EX)
-            for(int i=0; i<8; ++i)
-                GX->ub[2 * i + 1] = GX->ub[2 * i];
-        else
-            for(int i=0; i<8; ++i)
-                GX->ub[2 * i + 1] = EX->ub[i + 8];
+            GX->ub[2 * i + 1] = EX->ub[i + 8];
+        }
         break;
     case 0x69:  /* PUNPCKHWD Gx,Ex */
         nextop = F8;
         GETEX(0);
         GETGX;
-        for(int i=0; i<4; ++i)
+        for(int i=0; i<4; ++i) {
             GX->uw[2 * i] = GX->uw[i + 4];
-        if(GX==EX)
-            for(int i=0; i<4; ++i)
-                GX->uw[2 * i + 1] = GX->uw[2 * i];
-        else
-            for(int i=0; i<4; ++i)
-                GX->uw[2 * i + 1] = EX->uw[i + 4];
+            GX->uw[2 * i + 1] = EX->uw[i + 4];
+        }
         break;
     case 0x6A:  /* PUNPCKHDQ Gx,Ex */
         nextop = F8;
@@ -1682,6 +1715,38 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         for (int i=0; i<4; ++i)
             GX->ud[i] = (GX->ud[i]==EX->ud[i])?0xffffffff:0;
         break;
+    
+    case 0x78:  /* EXTRQ Ex, ib, ib */
+        // AMD only
+        nextop = F8;
+        if(!BOX64ENV(cputype) || (nextop&0xC0)>>3) {
+            #ifndef TEST_INTERPRETER
+            emit_signal(emu, SIGILL, (void*)R_RIP, 0);
+            #endif
+        } else {
+            GETEX(2);
+            tmp8u = F8&0x3f;
+            tmp8s = F8&0x3f;
+            EX->q[0]>>=tmp8u;
+            EX->q[0]&=((1<<(tmp8s+1))-1);
+        }
+        break;
+    case 0x79:  /* EXTRQ Ex, Gx */
+        // AMD only
+        nextop = F8;
+        if(!BOX64ENV(cputype) || !(MODREG)) {
+            #ifndef TEST_INTERPRETER
+            emit_signal(emu, SIGILL, (void*)R_RIP, 0);
+            #endif
+        } else {
+            GETGX;
+            GETEX(2);
+            tmp8u = GX->ub[0]&0x3f;
+            tmp8s = GX->ub[1]&0x3f;
+            EX->q[0]>>=tmp8u;
+            EX->q[0]&=((1<<(tmp8s+1))-1);
+        }
+        break;
 
     case 0x7C:  /* HADDPD Gx, Ex */
         nextop = F8;
@@ -1835,7 +1900,35 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         else
             EW->word[0] = shrd16(emu, EW->word[0], GW->word[0], tmp8u);
         break;
-
+    case 0xAE:                      /* Grp Ed (SSE) */
+        nextop = F8;
+        if(MODREG)
+            switch(nextop) {
+                default:
+                    return 0;
+            }
+        else
+        switch((nextop>>3)&7) {
+            case 6:                 /* CLWB Ed */
+                // same code and CLFLUSH, is it ok?
+                _GETED(0);
+                #if defined(DYNAREC) && !defined(TEST_INTERPRETER)
+                if(BOX64ENV(dynarec))
+                    cleanDBFromAddressRange((uintptr_t)ED, 8, 0);
+                #endif
+                break;
+            case 7:                 /* CLFLUSHOPT Ed */
+                // same code and CLFLUSH, is it ok?
+                _GETED(0);
+                #if defined(DYNAREC) && !defined(TEST_INTERPRETER)
+                if(BOX64ENV(dynarec))
+                    cleanDBFromAddressRange((uintptr_t)ED, 8, 0);
+                #endif
+                break;
+            default:
+                return 0;
+        }
+        break;
     case 0xAF:                      /* IMUL Gw,Ew */
         nextop = F8;
         GETEW(0);
@@ -2101,6 +2194,15 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         else
             GW->sword[0] = EB->sbyte[0];
         break;
+    case 0xBF:                      /* MOVSX Gw,Ew */
+        nextop = F8;
+        GETEW(0);
+        GETGW;
+        if(rex.w)
+            GW->sq[0] = EW->sword[0];
+        else
+            GW->sword[0] = EW->sword[0];
+        break;
 
     case 0xC1:                      /* XADD Gw,Ew */ // Xchange and Add
         nextop = F8;
@@ -2229,7 +2331,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         break;
     case 0xD6:                      /* MOVQ Ex,Gx */
         nextop = F8;
-        GETEX(0);
+        GETEX8(0);
         GETGX;
         EX->q[0] = GX->q[0];
         if(MODREG)
@@ -2363,11 +2465,11 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         nextop = F8;
         GETEX(0);
         GETGX;
-        if(isnan(EX->d[0]) || isinf(EX->d[0]) || EX->d[0]>0x7fffffff)
+        if(isnan(EX->d[0]) || isinf(EX->d[0]) || EX->d[0]>(double)0x7fffffff)
             GX->sd[0] = 0x80000000;
         else
             GX->sd[0] = EX->d[0];
-        if(isnan(EX->d[1]) || isinf(EX->d[1]) || EX->d[1]>0x7fffffff)
+        if(isnan(EX->d[1]) || isinf(EX->d[1]) || EX->d[1]>(double)0x7fffffff)
             GX->sd[1] = 0x80000000;
         else
             GX->sd[1] = EX->d[1];
