@@ -472,23 +472,21 @@ uintptr_t RunAVX_660F38(x64emu_t *emu, vex_t vex, uintptr_t addr, int *step)
             nextop = F8;
             GETEX(0);
             GETGX;
-            if(vex.l) {GETEY; GETGY;}
-            // ZF
-            u8 = 0;
-            for(int i=0; i<4 && !u8; ++i)
-                u8 |= ((EX->ud[i]>>31)&(GX->ud[i]>>31));
-            if(vex.l && !u8)
-                for(int i=0; i<4 && !u8; ++i)
-                    u8 |= ((EY->ud[i]>>31)&(GY->ud[i]>>31));
+            RESET_FLAGS(emu);
+            u8 = 0; tmp8u = 0;
+            for(int i=0; i<4; ++i) {
+                if((GX->ud[i]&EX->ud[i])>>31) u8 = 1;
+                if(((~GX->ud[i])&EX->ud[i])>>31) tmp8u = 1;
+            }
+            if(vex.l) {
+                GETEY; GETGY;
+                for(int i=0; i<4; ++i) {
+                    if((GY->ud[i]&EY->ud[i])>>31) u8 = 1;
+                    if(((~GY->ud[i])&EY->ud[i])>>31) tmp8u = 1;
+                }
+            }
             CONDITIONAL_SET_FLAG(!u8, F_ZF);
-            // CF
-            u8 = 0;
-            for(int i=0; i<4 && !u8; ++i)
-                u8 |= ((EX->ud[i]>>31)&((~GX->ud[i])>>31));
-            if(vex.l && !u8)
-                for(int i=0; i<4 && !u8; ++i)
-                    u8 |= ((EY->ud[i]>>31)&((~GY->ud[i])>>31));
-            CONDITIONAL_SET_FLAG(!u8, F_CF);
+            CONDITIONAL_SET_FLAG(!tmp8u, F_CF);
             CLEAR_FLAG(F_AF);
             CLEAR_FLAG(F_OF);
             CLEAR_FLAG(F_SF);
@@ -498,23 +496,21 @@ uintptr_t RunAVX_660F38(x64emu_t *emu, vex_t vex, uintptr_t addr, int *step)
             nextop = F8;
             GETEX(0);
             GETGX;
-            if(vex.l) {GETEY; GETGY;}
-            // ZF
-            u8 = 0;
-            for(int i=0; i<2 && !u8; ++i)
-                u8 |= ((EX->q[i]>>63)&(GX->q[i]>>63));
-            if(vex.l && !u8)
-                for(int i=0; i<2 && !u8; ++i)
-                    u8 |= ((EY->q[i]>>63)&(GY->q[i]>>63));
+            RESET_FLAGS(emu);
+            u8 = 0; tmp8u = 0;
+            for(int i=0; i<2; ++i) {
+                if((GX->q[i]&EX->q[i])>>63) u8 = 1;
+                if(((~GX->q[i])&EX->q[i])>>63) tmp8u = 1;
+            }
+            if(vex.l) {
+                GETEY; GETGY;
+                for(int i=0; i<2; ++i) {
+                    if((GY->q[i]&EY->q[i])>>63) u8 = 1;
+                    if(((~GY->q[i])&EY->q[i])>>63) tmp8u = 1;
+                }
+            }
             CONDITIONAL_SET_FLAG(!u8, F_ZF);
-            // CF
-            u8 = 0;
-            for(int i=0; i<2 && !u8; ++i)
-                u8 |= ((EX->q[i]>>63)&((~GX->q[i])>>63));
-            if(vex.l && !u8)
-                for(int i=0; i<2 && !u8; ++i)
-                    u8 |= ((EY->q[i]>>63)&((~GY->q[i])>>63));
-            CONDITIONAL_SET_FLAG(!u8, F_CF);
+            CONDITIONAL_SET_FLAG(!tmp8u, F_CF);
             CLEAR_FLAG(F_AF);
             CLEAR_FLAG(F_OF);
             CLEAR_FLAG(F_SF);
@@ -572,8 +568,8 @@ uintptr_t RunAVX_660F38(x64emu_t *emu, vex_t vex, uintptr_t addr, int *step)
             RESET_FLAGS(emu);
             if(vex.l) {
                 GETEY; GETGY;
-                CONDITIONAL_SET_FLAG(!(GY->u128&EY->u128), F_ZF);
-                CONDITIONAL_SET_FLAG(!((~GY->u128)&EY->u128), F_CF);
+                CONDITIONAL_SET_FLAG(!(GY->u128&EY->u128) && !(GX->u128&EX->u128), F_ZF);
+                CONDITIONAL_SET_FLAG(!((~GY->u128)&EY->u128) && !((~GX->u128)&EX->u128), F_CF);
             } else {
                 CONDITIONAL_SET_FLAG(!(GX->u128&EX->u128), F_ZF);
                 CONDITIONAL_SET_FLAG(!((~GX->u128)&EX->u128), F_CF);
