@@ -1,4 +1,5 @@
 #define _GNU_SOURCE
+#include <fenv.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -17,6 +18,7 @@
 #include "x64primop.h"
 #include "x64trace.h"
 #include "x87emu_private.h"
+#include "x87emu_setround.h"
 #include "box64context.h"
 #include "bridge.h"
 
@@ -34,6 +36,7 @@ uintptr_t RunD8(x64emu_t *emu, rex_t rex, uintptr_t addr, uintptr_t offs)
     #ifdef TEST_INTERPRETER
     x64emu_t*emu = test->emu;
     #endif
+    int oldround = fpu_setround(emu);
 
     nextop = F8;
     if(MODREG)
@@ -121,6 +124,7 @@ uintptr_t RunD8(x64emu_t *emu, rex_t rex, uintptr_t addr, uintptr_t offs)
             ST0.d = ST(nextop&7).d / ST0.d;
             break;
         default:
+            fesetround(oldround);
             return 0;
     } else
         switch((nextop>>3)&7) {
@@ -190,7 +194,9 @@ uintptr_t RunD8(x64emu_t *emu, rex_t rex, uintptr_t addr, uintptr_t offs)
                 ST0.d = *(float*)ED / ST0.d;
                 break;
             default:
+                fesetround(oldround);
                 return 0;
         }
-   return addr;
+    fesetround(oldround);
+    return addr;
 }
\ No newline at end of file
