@@ -290,6 +290,12 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 }
                 pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
+                if(BOX64ENV(dynarec_test)) {
+                    CLEAR_FLAG(F_OF);
+                    CLEAR_FLAG(F_SF);
+                    CLEAR_FLAG(F_AF);
+                    CLEAR_FLAG(F_PF);
+                }
                 break;
 
                 case 0xB0:                      /* CMPXCHG Eb,Gb */
@@ -472,6 +478,12 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                     }
                     pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
+                    if(BOX64ENV(dynarec_test)) {
+                        CLEAR_FLAG(F_OF);
+                        CLEAR_FLAG(F_SF);
+                        CLEAR_FLAG(F_AF);
+                        CLEAR_FLAG(F_PF);
+                    }
                     break;
 
                     case 0xBA:                      
@@ -494,6 +506,12 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                                     else
                                         CLEAR_FLAG(F_CF);
                                 }
+                                if(BOX64ENV(dynarec_test)) {
+                                    CLEAR_FLAG(F_OF);
+                                    CLEAR_FLAG(F_SF);
+                                    CLEAR_FLAG(F_AF);
+                                    CLEAR_FLAG(F_PF);
+                                }
                                 break;
                             case 5:             /* BTS Ed, Ib */
                                 CHECK_FLAGS(emu);
@@ -562,6 +580,12 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                                 }
                                 pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
+                                if(BOX64ENV(dynarec_test)) {
+                                    CLEAR_FLAG(F_OF);
+                                    CLEAR_FLAG(F_SF);
+                                    CLEAR_FLAG(F_AF);
+                                    CLEAR_FLAG(F_PF);
+                                }
                                 break;
                             case 6:             /* BTR Ed, Ib */
                                 CHECK_FLAGS(emu);
@@ -614,6 +638,12 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                                 }
                                 pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
+                                if(BOX64ENV(dynarec_test)) {
+                                    CLEAR_FLAG(F_OF);
+                                    CLEAR_FLAG(F_SF);
+                                    CLEAR_FLAG(F_AF);
+                                    CLEAR_FLAG(F_PF);
+                                }
                                 break;
                             case 7:             /* BTC Ed, Ib */
                                 CHECK_FLAGS(emu);
@@ -662,6 +692,12 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                                 }
                                 pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
+                                if(BOX64ENV(dynarec_test)) {
+                                    CLEAR_FLAG(F_OF);
+                                    CLEAR_FLAG(F_SF);
+                                    CLEAR_FLAG(F_AF);
+                                    CLEAR_FLAG(F_PF);
+                                }
                                 break;
 
                             default:
@@ -1125,6 +1161,42 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                     pthread_mutex_lock(&my_context->mutex_lock);
                     EB->byte[0] = not8(emu, EB->byte[0]);
                     pthread_mutex_unlock(&my_context->mutex_lock);
+#endif
+                    break;
+                default:
+                    return 0;
+            }
+            break;
+            case 0xF7:                      /* GRP3 Ed(,Id) */
+            nextop = F8;
+            tmp8u = (nextop>>3)&7;
+            GETED((tmp8u<2)?4:0);
+            switch(tmp8u) {
+                case 2:                 /* NOT Eb */
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
+                    if(rex.w)
+                        do {
+                            tmp64u = native_lock_read_dd(ED); 
+                            tmp64u = not64(emu, tmp64u);
+                        } while(native_lock_write_dd(ED, tmp64u));
+                    else {
+                        do {
+                            tmp32u = native_lock_read_d(ED); 
+                            tmp32u = not32(emu, tmp32u);
+                        } while(native_lock_write_d(ED, tmp32u));
+                        if(MODREG) ED->dword[1] = 0;
+                    }
+#else
+                    if(rex.w) {
+                        pthread_mutex_lock(&my_context->mutex_lock);
+                        ED->q[0] = not64(emu, ED->q[0]);
+                        pthread_mutex_unlock(&my_context->mutex_lock);
+                    } else {
+                        pthread_mutex_lock(&my_context->mutex_lock);
+                        ED->dword[0] = not32(emu, ED->dword[0]);
+                        pthread_mutex_unlock(&my_context->mutex_lock);
+                        if(MODREG) ED->dword[1] = 0;
+                    }
 #endif
                     break;
                 default:
