@@ -36,6 +36,7 @@ uintptr_t RunF20F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
     uint8_t tmp8u;
     int32_t tmp32s;
     int64_t tmp64s0, tmp64s1;
+    uint64_t tmp64u;
     reg64_t *oped, *opgd;
     sse_regs_t *opex, *opgx, eax1;
     mmx87_regs_t *opgm;
@@ -88,7 +89,12 @@ uintptr_t RunF20F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             GX->d[0] = ED->sdword[0];
         }
         break;
-
+    case 0x2B:  /* MOVNTSD Ex, Gx */
+        nextop = F8;
+        GETEX8(0);
+        GETGX;
+        EX->q[0] = GX->q[0];
+        break;
     case 0x2C:  /* CVTTSD2SI Gd, Ex */
         nextop = F8;
         _GETEX(0);
@@ -306,6 +312,41 @@ uintptr_t RunF20F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
         }
         break;
 
+    case 0x78:  /* INSERTQ Ex, Gx, ib, ib */
+        // AMD only
+        nextop = F8;
+        if(!BOX64ENV(cputype) || !(MODREG)) {
+            #ifndef TEST_INTERPRETER
+            emit_signal(emu, SIGILL, (void*)R_RIP, 0);
+            #endif
+        } else {
+            GETGX;
+            GETEX(2);
+            tmp8u = F8&0x3f;
+            tmp8s = F8&0x3f;
+            tmp64u = (1<<(tmp8s+1)-1);
+            EX->q[0] &=~(tmp64u<<tmp8u);
+            EX->q[0] |= (GX->q[0]&tmp64u)<<tmp8u;
+        }
+        break;
+    case 0x79:  /* INSERTQ Ex, Gx */
+        // AMD only
+        nextop = F8;
+        if(!BOX64ENV(cputype) || !(MODREG)) {
+            #ifndef TEST_INTERPRETER
+            emit_signal(emu, SIGILL, (void*)R_RIP, 0);
+            #endif
+        } else {
+            GETGX;
+            GETEX(2);
+            tmp8u = GX->ub[8]&0x3f;
+            tmp8s = GX->ub[9]&0x3f;
+            tmp64u = (1<<(tmp8s+1)-1);
+            EX->q[0] &=~(tmp64u<<tmp8u);
+            EX->q[0] |= (GX->q[0]&tmp64u)<<tmp8u;
+        }
+        break;
+
     case 0x7C:  /* HADDPS Gx, Ex */
         nextop = F8;
         _GETEX(0);
@@ -313,8 +354,7 @@ uintptr_t RunF20F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
         GX->f[0] += GX->f[1];
         GX->f[1] = GX->f[2] + GX->f[3];
         if(EX==GX) {
-            GX->f[2] = GX->f[0];
-            GX->f[3] = GX->f[1];
+            GX->q[1] = GX->q[0];
         } else {
             GX->f[2] = EX->f[0] + EX->f[1];
             GX->f[3] = EX->f[2] + EX->f[3];
@@ -327,8 +367,7 @@ uintptr_t RunF20F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
         GX->f[0] -= GX->f[1];
         GX->f[1] = GX->f[2] - GX->f[3];
         if(EX==GX) {
-            GX->f[2] = GX->f[0];
-            GX->f[3] = GX->f[1];
+            GX->q[1] = GX->q[0];
         } else {
             GX->f[2] = EX->f[0] - EX->f[1];
             GX->f[3] = EX->f[2] - EX->f[3];
