@@ -22,17 +22,23 @@
 #include "bridge.h"
 #include "signals.h"
 
-static pthread_mutex_t mutex_global_test;
-
 void print_banner(x64emu_t* ref)
 {
-    printf_log(LOG_NONE, "Warning, difference between %s Interpreter and Dynarec in %p (%02x %02x %02x %02x %02x %02x %02x %02x)\n"\
-        "=======================================\n",
-        (ref->segs[_CS]==0x23)?"x86":"x64" ,(void*)ref->old_ip,
-        ((uint8_t*)ref->old_ip)[0], ((uint8_t*)ref->old_ip)[1], ((uint8_t*)ref->old_ip)[2], ((uint8_t*)ref->old_ip)[3],
-        ((uint8_t*)ref->old_ip)[4], ((uint8_t*)ref->old_ip)[5], ((uint8_t*)ref->old_ip)[6], ((uint8_t*)ref->old_ip)[7]
-    );
-    printf_log(LOG_NONE, "DIFF: Dynarec |  Interpreter\n----------------------\n");
+    zydis_dec_t* dec = (ref->segs[_CS] == 0x23) ? my_context->dec32 : my_context->dec;
+
+    printf_log(LOG_NONE, "Warning, difference between %s Interpreter and Dynarec in %p",
+        (ref->segs[_CS] == 0x23) ? "x86" : "x64", (void*)ref->old_ip);
+    if (dec) {
+        printf_log_prefix(0, LOG_NONE, " (%s)\n", DecodeX64Trace(dec, ref->old_ip, 1));
+    } else {
+        printf_log_prefix(0, LOG_NONE, "(%02x %02x %02x %02x %02x %02x %02x %02x)\n",
+            ((uint8_t*)ref->old_ip)[0], ((uint8_t*)ref->old_ip)[1], ((uint8_t*)ref->old_ip)[2], ((uint8_t*)ref->old_ip)[3],
+            ((uint8_t*)ref->old_ip)[4], ((uint8_t*)ref->old_ip)[5], ((uint8_t*)ref->old_ip)[6], ((uint8_t*)ref->old_ip)[7]);
+    }
+    printf_log(LOG_NONE, "------------------------------------------------\n");
+    printf_log(LOG_NONE, "DIFF: Dynarec |  Interpreter\n");
+    printf_log(LOG_NONE, "------------------------------------------------\n");
+    // printf_log(LOG_NONE, "%s\n", DumpCPURegs(ref, ref->old_ip, ref->segs[_CS]==0x23));
 }
 #define BANNER if(!banner) {banner=1; print_banner(ref);}
 void x64test_check(x64emu_t* ref, uintptr_t ip)
@@ -108,7 +114,7 @@ void x64test_check(x64emu_t* ref, uintptr_t ip)
         BANNER;
         printf_log(LOG_NONE, "MXCSR: %x | %x\n", ref->mxcsr.x32, emu->mxcsr.x32);
     }
-    if(box64_avx)
+    if(BOX64ENV(avx))
         if(memcmp(ref->ymm, emu->ymm, sizeof(emu->ymm))) {
             BANNER;
             for(int i=0; i<16; ++i) {
@@ -130,11 +136,11 @@ void x64test_check(x64emu_t* ref, uintptr_t ip)
             BANNER;
             printf_log(LOG_NONE, "MEM: @%p :", (void*)test->memaddr);
             for(int i=0; i<test->memsize; ++i)
-                printf_log(LOG_NONE, " %02x", ((uint8_t*)test->memaddr)[i]);
-            printf_log(LOG_NONE, " |");
+                printf_log_prefix(0, LOG_NONE, " %02x", ((uint8_t*)test->memaddr)[i]);
+            printf_log_prefix(0, LOG_NONE, " |");
             for(int i=0; i<test->memsize; ++i)
-                printf_log(LOG_NONE, " %02x", test->mem[i]);
-            printf_log(LOG_NONE, "\n");
+                printf_log_prefix(0, LOG_NONE, " %02x", test->mem[i]);
+            printf_log_prefix(0, LOG_NONE, "\n");
         }
     }
     if(banner)  // there was an error, re-sync!
@@ -154,8 +160,6 @@ void x64test_step(x64emu_t* ref, uintptr_t ip)
         test->emu = NewX64Emu(my_context, ip, (uintptr_t)ref->init_stack, ref->size_stack, 0);
         CopyEmu(test->emu, ref);
     } else {
-        if (box64_dynarec_test == 2 && test->test && test->clean)
-            pthread_mutex_unlock(&mutex_global_test);
         // check if IP is same, else, sync
         uintptr_t prev_ip = test->emu->ip.q[0];
         if(test->clean)
@@ -167,20 +171,8 @@ void x64test_step(x64emu_t* ref, uintptr_t ip)
     // do a dry single step
     test->memsize = 0;
     test->clean = 1;
-    test->notest = 0;
     ref->old_ip = ip;
 
-    if (box64_dynarec_test == 2) pthread_mutex_lock(&mutex_global_test);
     RunTest(test);
-    if (box64_dynarec_test == 2 && test->notest) pthread_mutex_unlock(&mutex_global_test);
     // this will be analyzed next step
 }
-
-void x64test_init()
-{
-    pthread_mutexattr_t attr;
-    pthread_mutexattr_init(&attr);
-    pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
-    pthread_mutex_init(&mutex_global_test, &attr);
-    pthread_mutexattr_destroy(&attr);
-}
