@@ -10,8 +10,10 @@
 // 0 : doesn't exist, 1: Does exist
 int FileExist(const char* filename, int flags);
 
-// find a file, using Path if needed
+// find a file, using Path if needed, resolving symlinks
 char* ResolveFile(const char* filename, path_collection_t* paths);
+// find a file, using Path if needed, NOT resolving symlinks
+char* ResolveFileSoft(const char* filename, path_collection_t* paths);
 
 // 1: if file is an x86 elf, 0: if not (or not found)
 int FileIsX86ELF(const char* filename);
