@@ -9,4 +9,8 @@ void freeGLProcWrapper(box64context_t* context);
 
 void* getGLProcAddress(x64emu_t* emu, glprocaddress_t procaddr, const char* rname);
 
+#ifdef BOX32
+void* getGLProcAddress32(x64emu_t* emu, glprocaddress_t procaddr, const char* rname);
+#endif
+
 #endif //__GL_TOOLS_H__
\ No newline at end of file
