@@ -266,7 +266,7 @@ static int AddNeededLib_add(lib_t** maplib, int local, needed_libs_t* needed, in
                 return 1;
             }
             lm->l_addr = (Elf32_Addr)to_ptrv(GetElfDelta(lib->e.elf));
-            lm->l_name = to_ptrv(lib->name);
+            lm->l_name = to_cstring(lib->name);
             lm->l_ld = to_ptrv(GetDynamicSection(lib->e.elf));
         } else
         #endif
@@ -361,7 +361,7 @@ int AddNeededLib(lib_t* maplib, int local, int bindnow, int deepbind, needed_lib
             DecRefCount(&needed->libs[n-i-1], emu);
     }
     // all done
-    if(allow_missing_libs) return 0;
+    if(BOX64ENV(allow_missing_libs)) return 0;
     return ret;
 }
 EXPORTDYN
@@ -370,7 +370,7 @@ void RemoveNeededLib(lib_t* maplib, int local, needed_libs_t* needed, box64conte
     if(!needed) // no needed libs, no problems
         return;
     for(int i=0; i<needed->size; ++i) {
-        if(box64_log>=LOG_DEBUG && needed->libs[i])
+        if (BOX64ENV(log)>=LOG_DEBUG && needed->libs[i])
             printf_dump(LOG_DEBUG, "Will remove after failed init %s\n", needed->names[i]);
         AddNeededLib_remove(maplib, local, &needed->libs[i], box64, emu);
     }
@@ -462,6 +462,67 @@ int GetNoSelfSymbolStartEnd(lib_t *maplib, const char* name, uintptr_t* start, u
     // nope, not found
     return weak;
 }
+static int isLibPreloaded(library_t* lib)
+{
+    if(my_context->preload)
+        for(int i=0; i<my_context->preload->size; ++i)
+            if(my_context->preload->libs[i] == lib)
+                return 1;
+    return 0;
+}
+int GetNextSymbolStartEnd(lib_t *maplib, const char* name, uintptr_t* start, uintptr_t* end, elfheader_t* self, size_t size, int version, const char* vername, int veropt, void** elfsym)
+{
+    assert(self);   // need self for this one
+    int weak = 0;
+    int next = 0;
+    void* sym;
+    // search in needed libs from preloaded first, in order
+    if(my_context->preload)
+        for(int i=0; i<my_context->preload->size; ++i) {
+            if(next) {
+                if(GetLibGlobalSymbolStartEnd(my_context->preload->libs[i], name, start, end, size, &weak, &version, &vername, 0, &veropt, elfsym)) {
+                    return 1;
+                }
+                if(GetLibWeakSymbolStartEnd(my_context->preload->libs[i], name, start, end, size, &weak, &version, &vername, 0, &veropt, elfsym)) {
+                    return 1;
+                }
+            }
+            if(self==GetElf(my_context->preload->libs[i])) {
+                next = 1;
+            }
+        }
+    if(maplib==my_context->maplib) {
+        if(next) {
+            if((sym = ElfGetGlobalSymbolStartEnd(my_context->elfs[0], start, end, name, &version, &vername, 0, &veropt))) {
+                if(elfsym) *elfsym = sym;
+                return 1;
+            }
+            if((sym = ElfGetWeakSymbolStartEnd(my_context->elfs[0], start, end, name, &version, &vername, 0, &veropt))) {
+                if(elfsym) *elfsym = sym;
+                return 1;
+            }
+        }
+        if(self==my_context->elfs[0]) {
+            next = 1;
+        }
+    }
+    // search in global symbols
+    if(maplib) {
+        for(int i=0; i<maplib->libsz; ++i) if(!isLibPreloaded(maplib->libraries[i])) {
+            if(next) {
+                if(GetLibGlobalSymbolStartEnd(maplib->libraries[i], name, start, end, size, &weak, &version, &vername, 0, &veropt, elfsym)) {
+                    return 1;
+                }
+                if(GetLibWeakSymbolStartEnd(maplib->libraries[i], name, start, end, size, &weak, &version, &vername, 0, &veropt, elfsym)) {
+                    return 1;
+                }
+            }
+            if(self==GetElf(maplib->libraries[i]))
+                next = 1;
+        }
+    }
+    return 0;
+}
 static int GetGlobalSymbolStartEnd_internal(lib_t *maplib, const char* name, uintptr_t* start, uintptr_t* end, elfheader_t* self, int* version, const char** vername, int* veropt, void** elfsym)
 {
     int weak = 0;
@@ -524,13 +585,13 @@ int GetGlobalSymbolStartEnd(lib_t *maplib, const char* name, uintptr_t* start, u
     }
     #ifndef STATICBUILD
     // some special case symbol, defined inside box64 itself
-    if(!strcmp(name, "gdk_display") && !box64_nogtk) {
+    if(!strcmp(name, "gdk_display") && !BOX64ENV(nogtk)) {
         *start = (uintptr_t)my_GetGTKDisplay();
         *end = *start+sizeof(void*);
         printf_log(LOG_INFO, "Using global gdk_display for gdk-x11 (%p:%p)\n", start, *(void**)start);
         return 1;
     }
-    if(!strcmp(name, "g_threads_got_initialized") && !box64_nogtk) {
+    if(!strcmp(name, "g_threads_got_initialized") && !BOX64ENV(nogtk)) {
         *start = (uintptr_t)my_GetGthreadsGotInitialized();
         *end = *start+sizeof(int);
         printf_log(LOG_INFO, "Using global g_threads_got_initialized for gthread2 (%p:%p)\n", start, *(void**)start);
@@ -597,14 +658,14 @@ int GetGlobalWeakSymbolStartEnd(lib_t *maplib, const char* name, uintptr_t* star
     }
     #ifndef STATICBUILD
     // some special case symbol, defined inside box64 itself
-    if(!strcmp(name, "gdk_display") && !box64_nogtk) {
+    if(!strcmp(name, "gdk_display") && !BOX64ENV(nogtk)) {
         *start = (uintptr_t)my_GetGTKDisplay();
         *end = *start+sizeof(void*);
         if(elfsym) *elfsym = NULL;
         printf_log(LOG_INFO, "Using global gdk_display for gdk-x11 (%p:%p)\n", start, *(void**)start);
         return 1;
     }
-    if(!strcmp(name, "g_threads_got_initialized") && !box64_nogtk) {
+    if(!strcmp(name, "g_threads_got_initialized") && !BOX64ENV(nogtk)) {
         *start = (uintptr_t)my_GetGthreadsGotInitialized();
         *end = *start+sizeof(int);
         if(elfsym) *elfsym = NULL;
