@@ -74,6 +74,7 @@ GO("libxcb-icccm.so.4", libxcbicccm)
 GO("libxcb-util.so.1", libxcbutil)
 GO("libxcb-render-util.so.0", libxcbrenderutil)
 GO("libxcb-render.so.0", libxcbrender)
+GO("libxcb-res.so.0", libxcbres)
 GO("libxcb-sync.so.1", libxcbsync)
 GO("libxcb-xinerama.so.0", libxcbxinerama)
 GO("libxcb-xinput.so.0", libxcbxinput)
@@ -191,7 +192,8 @@ GO("libvulkan.so", vulkan)
 GO("libgbm.so.1", gbm)
 GO("libgomp.so.1", gomp)
 GO("libcap.so.2", cap)
-//GO("libnsl.so.1", nsl)
+GO("libpcap.so.0.8", pcap)
+GO("libpcap.so.0", pcap)
 GO("libkrb5.so.3", krb5)
 GO("libgssapi_krb5.so.2", gssapikrb5)
 GO("libgssapi.so.3", gssapi)
