@@ -6,7 +6,7 @@ GO("libc.musl-x86_64.so.1", libcmusl)
 #ifdef ANDROID
 GO("libpthread.so", libpthread)
 GO("librt.so", librt)
-GO("libbsd.so", libbsd)
+//GO("libbsd.so", libbsd)
 GO("libc.so", libc)
 GO("libm.so", libm)
 GO("libdl.so", libdl)
@@ -14,7 +14,7 @@ GO("libresolv.so", libresolv)
 #else
 GO("libpthread.so.0", libpthread)
 GO("librt.so.1", librt)
-GO("libbsd.so.0", libbsd)
+//GO("libbsd.so.0", libbsd)
 GO("libc.so.6", libc)
 GO("libm.so.6", libm)
 GO("libdl.so.2", libdl)
