@@ -286,7 +286,7 @@ GO(cairo_set_source_rgb, vFpddd)
 GO(cairo_set_source_rgba, vFpdddd)
 GO(cairo_set_source_surface, vFppdd)
 GO(cairo_set_tolerance, vFpd)
-//GOM(cairo_set_user_data, iFEpppp)
+GOM(cairo_set_user_data, iFEpppp)
 GO(cairo_show_glyphs, vFppi)
 GO(cairo_show_page, vFp)
 GO(cairo_show_text, vFpp)
@@ -332,7 +332,7 @@ GO(cairo_surface_set_device_offset, vFpdd)
 GO(cairo_surface_set_device_scale, vFpdd)
 GO(cairo_surface_set_fallback_resolution, vFpdd)
 //GOM(cairo_surface_set_mime_data, iFEpppLpp)
-//GOM(cairo_surface_set_user_data, iFEpppp)
+GOM(cairo_surface_set_user_data, iFEpppp)
 GO(cairo_surface_show_page, vFp)
 GO(cairo_surface_status, uFp)
 GO(cairo_surface_supports_mime_type, iFpp)
@@ -371,16 +371,16 @@ GO(cairo_xcb_device_debug_cap_xrender_version, vFpii)
 //GO(cairo_xcb_device_debug_cap_xshm_version, 
 GO(cairo_xcb_device_debug_get_precision, iFp)
 GO(cairo_xcb_device_debug_set_precision, vFpi)
-GO(cairo_xcb_device_get_connection, pFp)
-GO(cairo_xcb_surface_create, pFpupii)
-GO(cairo_xcb_surface_create_for_bitmap, pFppuii)
-GO(cairo_xcb_surface_create_with_xrender_format, pFppupii)
+GOM(cairo_xcb_device_get_connection, pFEp)
+GO(cairo_xcb_surface_create, pFbupii)
+GO(cairo_xcb_surface_create_for_bitmap, pFbpuii)
+GO(cairo_xcb_surface_create_with_xrender_format, pFbpupii)
 GO(cairo_xcb_surface_set_drawable, vFpuii)
 GO(cairo_xcb_surface_set_size, vFpii)
 GO(cairo_xlib_device_debug_cap_xrender_version, vFpii)
 GO(cairo_xlib_device_debug_get_precision, iFp)
 GO(cairo_xlib_device_debug_set_precision, vFpi)
-//GO(cairo_xlib_surface_create, 
+GO(cairo_xlib_surface_create, pFpppii)
 GO(cairo_xlib_surface_create_for_bitmap, pFpLpii)
 GO(cairo_xlib_surface_create_with_xrender_format, pFpLppii)
 //GO(cairo_xlib_surface_get_depth, 
