@@ -21,7 +21,7 @@ const char* crashhandlerName = "crashhandler.so";
 #define LIBNAME crashhandler
 
 #define PRE_INIT                                                \
-    if(!box64_dummy_crashhandler)                               \
+    if(!BOX64ENV(dummy_crashhandler))                           \
         return -1;                                              \
     if(1)                                                       \
         lib->w.lib = dlopen(NULL, RTLD_LAZY | RTLD_GLOBAL);\
