@@ -99,7 +99,7 @@ GO(dbus_message_copy, pFp)
 GO(dbus_message_demarshal, pFpip)
 GO(dbus_message_demarshal_bytes_needed, iFpi)
 GO(dbus_message_free_data_slot, vFp)
-GOM(dbus_message_get_args, iFEppip)
+GOM(dbus_message_get_args, iFEppiV)
 GO(dbus_message_get_allow_interactive_authorization, iFp)
 GOM(dbus_message_get_args_valist, iFEppiA)
 GO(dbus_message_get_auto_start, iFp)
