@@ -18,11 +18,8 @@
 #include "myalign.h"
 #include "bridge.h"
 
-#ifdef ANDROID
-    const char* flacName = "libFLAC.so";
-#else
-    const char* flacName = "libFLAC.so.8";
-#endif
+const char* flacName = "libFLAC.so.8";
+#define ALTNAME "libFLAC.so"
 
 #define LIBNAME flac
 
@@ -347,12 +344,4 @@ EXPORT int my_FLAC__stream_decoder_init_stream(x64emu_t* emu, void* decoder,
     return ret;
 }
 
-
-#define CUSTOM_INIT \
-    getMy(lib);
-
-#define CUSTOM_FINI \
-    freeMy();
-
 #include "wrappedlib_init.h"
-
