@@ -291,6 +291,7 @@ GO(g_date_time_compare, iFpp)
 GO(g_date_time_difference, lFpp)
 GO(g_date_time_equal, iFpp)
 GO(g_date_time_format, pFpp)
+GO(g_date_time_format_iso8601, pFp)
 GO(g_date_time_get_day_of_month, iFp)
 GO(g_date_time_get_day_of_week, iFp)
 GO(g_date_time_get_day_of_year, iFp)
@@ -708,6 +709,7 @@ GO(g_mem_chunk_new, pFpiLi)
 GO(g_mem_chunk_print, vFp)
 GO(g_mem_chunk_reset, vFp)
 GO(g_memdup, pFpu)
+GO(g_memdup2, pFpL)
 GO(g_mem_is_system_malloc, iFv)
 GO(g_mem_profile, vFv)
 //GOM(g_mem_set_vtable, vFEp)
@@ -859,7 +861,7 @@ GO(g_queue_insert_after, vFppp)
 GO(g_queue_insert_after_link, vFppp)    // 2.62+
 GO(g_queue_insert_before, vFppp)
 GO(g_queue_insert_before_link, vFppp)   // 2.62+
-//GOM(g_queue_insert_sorted, vFEpppp)
+GOM(g_queue_insert_sorted, vFEpppp)
 GO(g_queue_is_empty, iFp)
 GO(g_queue_link_index, iFpp)
 GO(g_queue_new, pFv)
@@ -988,7 +990,7 @@ GO(g_sequence_get_end_iter, pFp)
 GO(g_sequence_get_iter_at_pos, pFpi)
 GO(g_sequence_get_length, iFp)
 GO(g_sequence_insert_before, pFpp)
-//GOM(g_sequence_insert_sorted, pFpppp)
+GOM(g_sequence_insert_sorted, pFEpppp)
 //GOM(g_sequence_insert_sorted_iter, pFpppp)
 GO(g_sequence_iter_compare, iFpp)
 GO(g_sequence_iter_get_position, iFp)
@@ -998,11 +1000,11 @@ GO(g_sequence_iter_is_end, iFp)
 GO(g_sequence_iter_move, pFpi)
 GO(g_sequence_iter_next, pFp)
 GO(g_sequence_iter_prev, pFp)
-//GOM(g_sequence_lookup, pFpppp)
+GOM(g_sequence_lookup, pFEpppp)
 //GOM(g_sequence_lookup_iter, pFpppp)
 GO(g_sequence_move, vFpp)
 GO(g_sequence_move_range, vFppp)
-//GOM(g_sequence_new, pFp)
+GOM(g_sequence_new, pFEp)
 GO(g_sequence_prepend, pFpp)
 GO(g_sequence_range_get_midpoint, pFpp)
 GO(g_sequence_remove, vFp)
@@ -1326,13 +1328,13 @@ GO(g_trash_stack_peek, pFp)
 GO(g_trash_stack_pop, pFp)
 GO(g_trash_stack_push, vFpp)
 GO(g_tree_destroy, vFp)
-//GOM(g_tree_foreach, vFppp)
+GOM(g_tree_foreach, vFEppp)
 GO(g_tree_height, iFp)
 GO(g_tree_insert, vFppp)
 GO(g_tree_lookup, pFpp)
 GO(g_tree_lookup_extended, iFpppp)
-//GOM(g_tree_new, pFp)
-//GOM(g_tree_new_full, pFpppp)
+GOM(g_tree_new, pFEp)
+GOM(g_tree_new_full, pFEpppp)
 //GOM(g_tree_new_with_data, pFpp)
 GO(g_tree_nnodes, iFp)
 GO(g_tree_ref, pFp)
