@@ -21,15 +21,17 @@
 const char* libcmuslName = "libc.musl-x86_64.so.1";
 #define LIBNAME libcmusl
 
+#ifndef STATICBUILD
 #define PRE_INIT\
     if(1)                                                           \
         lib->w.lib = dlopen(NULL, RTLD_LAZY | RTLD_GLOBAL);    \
     else
+#endif
 
 #define CUSTOM_INIT \
-    box64_musl = 1; \
-    setNeededLibs(lib, 4, "libc.so.6", "libdl.so.2", "libm.so.6", "libpthread.so.0");
+    box64_musl = 1;
+
+#define NEEDED_LIBS "libc.so.6", "libdl.so.2", "libm.so.6", "libpthread.so.0"
 
 // define all standard library functions
 #include "wrappedlib_init.h"
-
