@@ -64,11 +64,6 @@ EXPORT int my6_tputs(x64emu_t* emu, void* str, int affcnt, void* f)
     return my->tputs(str, affcnt, find_putc_Fct(f));
 }
 
-#define CUSTOM_INIT \
-    SETALT(my6_); \
-    getMy(lib);
-
-#define CUSTOM_FINI \
-    freeMy();
+#define ALTMY my6_
 
 #include "wrappedlib_init.h"
