@@ -6,11 +6,11 @@
 //GO(xcb_present_event_next, 
 DATA(xcb_present_id, 8)
 //GO(xcb_present_notify_end, 
-GO(xcb_present_notify_msc, pFpuuUUU)
+GO(xcb_present_notify_msc, pFbuuUUU)
 //GO(xcb_present_notify_msc_checked, 
 //GO(xcb_present_notify_next, 
-GO(xcb_present_pixmap, pFpuuuuuwwuuuuUUUup)
-GO(xcb_present_pixmap_checked, pFpuuuuuwwuuuuUUUup)
+GO(xcb_present_pixmap, pFbuuuuuwwuuuuUUUup)
+GO(xcb_present_pixmap_checked, pFbuuuuuwwuuuuUUUup)
 //GO(xcb_present_pixmap_notifies, 
 //GO(xcb_present_pixmap_notifies_iterator, 
 //GO(xcb_present_pixmap_notifies_length, 
@@ -18,12 +18,12 @@ GO(xcb_present_pixmap_checked, pFpuuuuuwwuuuuUUUup)
 //GO(xcb_present_query_capabilities, 
 //GO(xcb_present_query_capabilities_reply, 
 //GO(xcb_present_query_capabilities_unchecked, 
-GO(xcb_present_query_version, pFpuu)
-GO(xcb_present_query_version_reply, pFpup)
-//GO(xcb_present_query_version_unchecked, 
+GO(xcb_present_query_version, pFbuu)
+GO(xcb_present_query_version_reply, pFbup)
+GO(xcb_present_query_version_unchecked, pFbuu)
 //GO(xcb_present_redirect_notify_notifies, 
 //GO(xcb_present_redirect_notify_notifies_iterator, 
 //GO(xcb_present_redirect_notify_notifies_length, 
 //GO(xcb_present_redirect_notify_sizeof, 
-//GO(xcb_present_select_input, 
-GO(xcb_present_select_input_checked, pFpuuu)
+GO(xcb_present_select_input, pFbuuu)
+GO(xcb_present_select_input_checked, pFbuuu)
