@@ -1578,7 +1578,7 @@ void my_autobridge_mainloop_api(x64emu_t* emu, void* api)
 }
 
 #define PRE_INIT        \
-    if(box64_nopulse)   \
+    if(BOX64ENV(nopulse))   \
         return -1;
 
 #include "wrappedlib_init.h"
