@@ -8,21 +8,21 @@ GO(TTF_FontAscent, iFp)
 GO(TTF_FontDescent, iFp)
 GO(TTF_FontFaceFamilyName, pFp)
 GO(TTF_FontFaceIsFixedWidth, iFp)
-GO(TTF_FontFaces, iFp)
+GO(TTF_FontFaces, lFp)
 GO(TTF_FontFaceStyleName, pFp)
 GO(TTF_FontHeight, iFp)
 GO(TTF_FontLineSkip, iFp)
 GO(TTF_GetFontHinting, iFp)
 GO(TTF_GetFontKerning, iFp)
-GO(TTF_GetFontKerningSize, iFp)
+GO(TTF_GetFontKerningSize, iFpii)
 GO(TTF_GetFontOutline, iFp)
 GO(TTF_GetFontStyle, iFp)
-GO(TTF_GlyphIsProvided, iFpu)
-GO(TTF_GlyphMetrics, iFpuppppp)
+GO(TTF_GlyphIsProvided, iFpW)
+GO(TTF_GlyphMetrics, iFpWppppp)
 GO(TTF_Init, iFv)
 GO(TTF_Linked_Version, pFv)
 GO(TTF_OpenFont, pFpi)
-GO(TTF_OpenFontIndex, pFpii)
+GO(TTF_OpenFontIndex, pFpil)
 GOM(TTF_OpenFontIndexRW, pFEpiii)
 GOM(TTF_OpenFontRW, pFEpii)
 GO(TTF_Quit, vFv)
