@@ -8,7 +8,7 @@
 DATAB(_qfree, 4) // Warning: failed to confirm
 
 GO(XActivateScreenSaver, iFX)
-//GOM(XAddConnectionWatch, iFEXpp)
+GOM(XAddConnectionWatch, iFEXpp)
 GO(XAddExtension, pFX)
 GO(XAddHost, iFXbiip_)
 GO(XAddHosts, iFXbiip_i)
@@ -33,7 +33,7 @@ GO(XAllPlanes, LFv)
 //GO(_XAsyncErrorHandler, iFpppip)
 GO(XAutoRepeatOff, iFX)
 GO(XAutoRepeatOn, iFX)
-GO(XBaseFontNameListOfFontSet, pFp)
+GOM(XBaseFontNameListOfFontSet, pFEp)
 GO(XBell, iFXi)
 GO(XBitmapBitOrder, iFX)
 GO(XBitmapPad, iFX)
@@ -50,7 +50,7 @@ GOM(XChangeProperty, iFEXLLLiipi)
 GO(XChangeSaveSet, iFXLi)
 GOM(XChangeWindowAttributes, iFEXLLp)
 GOM(XCheckIfEvent, iFEXppp)
-//GO(XCheckMaskEvent, iFplp)
+GOM(XCheckMaskEvent, iFEXlp)
 GOM(XCheckTypedEvent, iFEXip)
 GOM(XCheckTypedWindowEvent, iFEXLip)
 GOM(XCheckWindowEvent, iFEXLlp)
@@ -198,8 +198,8 @@ GO(XcmsPrefixOfFormat, pFL)
 GO(XConfigureWindow, iFXLuriiiiiLi_)
 GO(XConnectionNumber, iFX)
 //GO(_XConnectXCB, 
-GO(XContextDependentDrawing, iFp)
-GO(XContextualDrawing, iFp)
+GOM(XContextDependentDrawing, iFEp)
+GOM(XContextualDrawing, iFEp)
 GO(XConvertCase, vFLbL_bL_)
 GO(XConvertSelection, iFXLLLLL)
 GO(XCopyArea, iFXLLpiiuuii)
@@ -249,14 +249,14 @@ GO(XDeleteContext, iFXLi)
 //GO(XDeleteModifiermapEntry, bip_Fbip_Ci)
 GO(XDeleteProperty, iFXLL)
 //GOM(_XDeq, vFEXpp)
-//GOM(_XDeqAsyncHandler, vFEXp)
+GOM(_XDeqAsyncHandler, vFEXp)
 GO(XDestroyIC, vFp)
 GOM(XDestroyImage, iFEp)  //need to unbridge
 GO(XDestroyOC, vFp)
 GO(XDestroyRegion, iFp)
 GO(XDestroySubwindows, iFXL)
 GO(XDestroyWindow, iFXL)
-GO(XDirectionalDependentDrawing, iFp)
+GOM(XDirectionalDependentDrawing, iFEp)
 GO(XDisableAccessControl, iFX)
 GO(XDisplayCells, iFXi)
 GO(XDisplayHeight, iFXi)
@@ -319,7 +319,7 @@ GO(XEventsQueued, iFXi)
 //DATA(_Xevent_to_mask, 
 //GO(_XEventToWire, iFppp)
 GO(XExtendedMaxRequestSize, lFX)
-GO(XExtentsOfFontSet, pFp)
+GOM(XExtentsOfFontSet, pFEp)
 //GO(_XF86BigfontFreeFontMetrics, 
 //GO(_XF86LoadQueryLocaleFont, iFpppp)
 GO(XFetchBuffer, pFXpi)
@@ -338,7 +338,7 @@ GO(_XFlush, vFX)
 GO(XFlush, iFX)
 GO(XFlushGC, vFXp)
 GO(_XFlushGCCache, vFXp)
-//GO(XFontsOfFontSet, iFppp)
+GOM(XFontsOfFontSet, iFEppp)
 GO(XForceScreenSaver, iFXi)
 GO(XFree, iFp)
 //GO(_XFreeAtomTable, 
@@ -355,7 +355,7 @@ GOM(XFreeFont, iFEXp)
 GOM(XFreeFontInfo, iFEppi)
 GOM(XFreeFontNames, iFEp)
 //GO(XFreeFontPath, iFbp_)
-GO(XFreeFontSet, vFXp)
+GOM(XFreeFontSet, vFEXp)
 GO(XFreeGC, iFXp)
 GOM(XFreeModifiermap, iFEp)
 //DATAM(_XFreeMutex_fn, 4)
@@ -367,7 +367,7 @@ GO(XFreeThreads, iFv)
 GO(XGContextFromGC, LFp)
 GO(XGeometry, iFXippuuuiipppp)
 GO(_XGetAsyncData, vFXppiiii)
-//GO(_XGetAsyncReply, pFppppiii)
+GO(_XGetAsyncReply, pFppppiii)
 GO(XGetAtomName, pFXL)
 GO(XGetAtomNames, iFXbL_ibp_)
 //GO(_XGetBitsPerPixel, 
@@ -392,7 +392,7 @@ GO(XGetKeyboardControl, iFXbiiuuLip_)
 GO(XGetKeyboardMapping, pFXCip) // return a KeySym*, so an unsigned long*
 //GO(_XGetLCValues, 
 GOM(XGetModifierMapping, pFEX)
-//GO(XGetMotionEvents, bLww_FXLLLp)
+GOM(XGetMotionEvents, pFEXLLLp)
 //GO(XGetNormalHints, iFpLp)
 //GO(XGetOCValues, pFpppppppppp) // use varargs
 //GOM(XGetOMValues, pFEpV)
@@ -400,7 +400,7 @@ GOM(XGetPixel, LFEXii)  // need unbridging
 GO(dummy_XGetPixel, LFpii)     // for the wrapper
 GO(XGetPointerControl, iFXppp)
 GO(XGetPointerMapping, iFXpi)
-//GOM(_XGetRequest, pFEXCL)
+GOM(_XGetRequest, pFEXCL)
 //GO(XGetRGBColormaps, iFXLbbLLLLLLLLLL__pL)
 //GO(_XGetScanlinePad, 
 GO(XGetScreenSaver, iFXpppp)
@@ -415,12 +415,12 @@ GOM(XGetVisualInfo, pFEXlpp)
 GOM(XGetWindowAttributes, iFEXLp)
 GOM(XGetWindowProperty, iFEXLLlliLppppp)
 GO(XGetWMClientMachine, iFXLbpLiL_)
-//GO(XGetWMColormapWindows, iFXLbbL__p)
+GOM(XGetWMColormapWindows, iFEXLpp)
 GOM(XGetWMHints, pFEXL)
 GO(XGetWMIconName, iFXLbpLiL_)
 GO(XGetWMName, iFXLbpLiL_)
 GOM(XGetWMNormalHints, iFEXLpp)
-//GO(XGetWMProtocols, iFXLbbL__p)
+GOM(XGetWMProtocols, iFEXLpp)
 //GO(XGetWMSizeHints, iFpLppL)
 //GO(XGetZoomHints, iFpLp)
 DATAB(_Xglobal_lock, 4)
@@ -682,7 +682,7 @@ GO(XkbForceDeviceBell, iFXiiii)
 //GO(XkbFreeGeomSections, vFpiii)
 //GO(XkbFreeGeomShapes, vFpiii)
 //GO(XkbFreeIndicatorMaps, vFp)
-//GO(XkbFreeKeyboard, vFpui)
+GOM(XkbFreeKeyboard, vFEpui)
 //GO(XkbFreeNames, vFpui)
 //GO(_XkbFreeReadBuffer, 
 //GO(XkbFreeServerMap, vFpui)
@@ -691,7 +691,7 @@ GO(XkbGetAutoRepeatRate, iFXupp)
 GO(XkbGetAutoResetControls, iFXpp)
 //GO(_XkbGetCharset, 
 //GO(XkbGetCompatMap, iFpup)
-//GO(XkbGetControls, iFpLp)
+GOM(XkbGetControls, iFEXLp)
 //GO(_XkbGetConverters, 
 GO(XkbGetDetectableAutoRepeat, iFXp)
 //GO(XkbGetDeviceButtonActions, iFppiuu)
@@ -710,17 +710,17 @@ GO(XkbGetIndicatorState, iFXup)
 //GO(XkbGetKeySyms, iFpuup)
 //GO(XkbGetKeyTypes, iFpuup)
 //GO(XkbGetKeyVirtualModMap, iFpuup)
-//GOSM(XkbGetMap, bXWWCCpbWWppppCCCCCCCCCCCCCCCCp_bCCbpCCppLbL__WWbL_pp_bLpppppppppppppppppppppppppppppppp_bLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLppbL_LCCW_bbLCCCCp_ppppWW_bLWWpbup_bup_WWWWWWWWWWWWbpp_bup_bLWWbWWWp_bWWWp_bWWWp_p_bLCwwWWwWWWWWWbwwWWipp_bppppp_pp_bppppp_p__FEXuu)
+GOM(XkbGetMap, pFEXuu)
 //GO(XkbGetMapChanges, iFppp)
 GO(XkbGetNamedDeviceIndicator, iFXuuuLpppp)
 //GO(XkbGetNamedGeometry, iFppL)
 GO(XkbGetNamedIndicator, iFXLpppp)
-//GO(XkbGetNames, iFpup)
+GOM(XkbGetNames, iFEXup)
 GO(XkbGetPerClientControls, iFXp)
 //GO(_XkbGetReadBufferCountedString, 
 //GO(_XkbGetReadBufferPtr, 
 GO(XkbGetState, iFXup)
-//GO(XkbGetUpdatedMap, iFpup)
+GOM(XkbGetUpdatedMap, iFEXup)
 //GO(XkbGetVirtualMods, iFpup)
 GO(XkbGetXlibControls, uFX)
 GO(XkbIgnoreExtension, iFi)
@@ -873,7 +873,7 @@ GO(XListPixmapFormats, pFXp)
 GO(XLoadFont, LFXp)
 GOM(XLoadQueryFont, pFEXp)
 GO(xlocaledir, vFpi)
-GO(XLocaleOfFontSet, pFp)
+GOM(XLocaleOfFontSet, pFEp)
 GO(XLocaleOfIM, pFp)
 GO(XLocaleOfOM, pFp)
 GOM(XLockDisplay, vFEX)
@@ -896,9 +896,9 @@ GO(XMaxRequestSize, lFX)
 //GO(_XmbDefaultTextEscapement, 
 //GO(_XmbDefaultTextExtents, 
 //GO(_XmbDefaultTextPerCharExtents, 
-GO(XmbDrawImageString, vFXLppiipi)
-GO(XmbDrawString, vFXLppiipi)
-GO(XmbDrawText, vFXLpiibpiip_i)
+GOM(XmbDrawImageString, vFEXLppiipi)
+GOM(XmbDrawString, vFEXLppiipi)
+GOM(XmbDrawText, vFEXLpiipi)
 //GO(_XmbGenericDrawImageString, 
 //GO(_XmbGenericDrawString, 
 //GO(_XmbGenericTextEscapement, 
@@ -907,14 +907,14 @@ GO(XmbDrawText, vFXLpiibpiip_i)
 GO(_Xmblen, iFpi)
 GOM(XmbLookupString, iFEpppipp)
 GO(XmbResetIC, pFp)
-//GO(XmbSetWMProperties, vFpLpppippp)
+GOM(XmbSetWMProperties, vFEXLpppippp)
 //GO(_Xmbstoutf8, 
 //GO(_Xmbstowcs, 
-GO(XmbTextEscapement, iFppi)
-GO(XmbTextExtents, iFppipp)
+GOM(XmbTextEscapement, iFEppi)
+GOM(XmbTextExtents, iFEppipp)
 //GO(_XmbTextListToTextProperty, 
 GOM(XmbTextListToTextProperty, iFEXpiup)
-GO(XmbTextPerCharExtents, iFppippippp)
+GOM(XmbTextPerCharExtents, iFEppippippp)
 //GO(_XmbTextPropertyToTextList, 
 GOM(XmbTextPropertyToTextList, iFEXbpLiL_pp)
 GO(_Xmbtowc, iFppi)
@@ -943,7 +943,7 @@ GO(XOpenOM, pFXppp)
 GO(XParseColor, iFXLpbLWWWcc_)
 GO(XParseGeometry, iFppppp)
 GOM(XPeekEvent, iFEXp) // Warning: failed to confirm
-//GOM(XPeekIfEvent, iFEpppp)
+GOM(XPeekIfEvent, iFEpppp)
 GO(XPending, iFX)
 GO(Xpermalloc, pFu)
 //GOM(XPlanesOfScreen, iFEbpXLiiiiipippLLLiiiil_)
@@ -958,7 +958,7 @@ GO(XProcessInternalConnection, vFXi)
 GO(XProtocolRevision, iFX)
 GO(XProtocolVersion, iFX)
 //GO(_XPutBackEvent, iFpp)
-//GO(XPutBackEvent, iFpp)
+GOM(XPutBackEvent, iFEXp)
 GOM(XPutImage, iFEXLppiiiiuu)
 GO(XPutPixel, vFXiiL)
 GO(XQLength, iFX)
@@ -996,7 +996,7 @@ GO(XRemoveFromSaveSet, iFXL)
 GO(XRemoveHost, iFXbiip_)
 GO(XRemoveHosts, iFXbiip_i)
 GO(XReparentWindow, iFXLLii)
-GO(_XReply, iFXpii)
+GOM(_XReply, iFEXpii)
 GO(XResetScreenSaver, iFX)
 GO(XResizeWindow, iFXLuu)
 GO(XResourceManagerString, pFX)
@@ -1039,7 +1039,7 @@ GO(XRotateBuffers, iFXi)
 GO(XRotateWindowProperties, iFXLbL_ii)
 GO(XSaveContext, iFXLip)
 GO(XScreenCount, iFX)
-//GOM(XScreenNumberOfScreen, iFEbpXLiiiiipippLLLiiiil_)
+GOM(XScreenNumberOfScreen, iFEp)
 //GOM(XScreenOfDisplay, bpXLiiiiipippLLLiiiil_FEXi)
 //GOM(_XScreenOfWindow, bpXLiiiiipippLLLiiiil_FEXL)
 //GOM(XScreenResourceString, pFEbpXLiiiiipippLLLiiiil_)
@@ -1114,7 +1114,7 @@ GO(XSetWMName, vFXLrpLiL_)
 GOM(XSetWMNormalHints, vFEXLp)
 GOM(XSetWMProperties, vFEXLpppippp)
 GOM(XSetWMProtocols, iFEXLpi)
-//GO(XSetWMSizeHints, vFpLpL)
+GOM(XSetWMSizeHints, vFEXLpL)
 //GO(XSetZoomHints, iFpLp)
 GO(XShrinkRegion, iFpii)
 GO(XStoreBuffer, iFXpii)
@@ -1133,11 +1133,11 @@ GO(XSupportsLocale, iFv)
 GO(XSync, iFXi)
 GOM(XSynchronize, pFEXi)
 GOM(XTextExtents, iFEppipppp)
-//GO(XTextExtents16, iFppipppp)
+GOM(XTextExtents16, iFEppipppp)
 //GO(_XTextHeight, iFppi)
 //GO(_XTextHeight16, iFppi)
 //GO(XTextPropertyToStringList, iFbpLiL_bbp__p)
-//GO(XTextWidth, iFppi)
+GOM(XTextWidth, iFEppi)
 GOM(XTextWidth16, iFEppi)
 //DATAB(_Xthread_self_fn, 4)
 GO(XTranslateCoordinates, iFXLLiippBL_)
@@ -1174,9 +1174,9 @@ GO(XUnsetICFocus, vFp)
 //GO(_Xutf8DefaultTextEscapement, 
 //GO(_Xutf8DefaultTextExtents, 
 //GO(_Xutf8DefaultTextPerCharExtents, 
-GO(Xutf8DrawImageString, vFXLppiipi)
-GO(Xutf8DrawString, vFXLppiipi)
-GO(Xutf8DrawText, vFXLpiibpiip_i)
+GOM(Xutf8DrawImageString, vFEXLppiipi)
+GOM(Xutf8DrawString, vFEXLppiipi)
+GOM(Xutf8DrawText, vFEXLpiipi)
 //GO(_Xutf8GenericDrawImageString, 
 //GO(_Xutf8GenericDrawString, 
 //GO(_Xutf8GenericTextEscapement, 
@@ -1185,13 +1185,13 @@ GO(Xutf8DrawText, vFXLpiibpiip_i)
 GOM(Xutf8LookupString, iFEpppipp)
 GO(Xutf8ResetIC, pFp)
 GOM(Xutf8SetWMProperties, vFEXLpppippp) // Warning: failed to confirm
-GO(Xutf8TextEscapement, iFppi)
-GO(Xutf8TextExtents, iFppipp)
+GOM(Xutf8TextEscapement, iFEppi)
+GOM(Xutf8TextExtents, iFEppipp)
 //GO(_Xutf8TextListToTextProperty, 
 GOM(Xutf8TextListToTextProperty, iFEXpiup)
-GO(Xutf8TextPerCharExtents, iFppippippp)
+GOM(Xutf8TextPerCharExtents, iFEppippippp)
 //GO(_Xutf8TextPropertyToTextList, 
-//GO(Xutf8TextPropertyToTextList, iFXbpLiL_bbp__p)
+GOM(Xutf8TextPropertyToTextList, iFEXbpLiL_pp)
 GOM(XVaCreateNestedList, pFEiV)
 GO(XVendorRelease, iFX)
 GO(_XVIDtoVisual, pFXL)
@@ -1202,9 +1202,9 @@ GO(XWarpPointer, iFXLLiiuuii)
 //GO(_XwcDefaultTextEscapement, 
 //GO(_XwcDefaultTextExtents, 
 //GO(_XwcDefaultTextPerCharExtents, 
-GO(XwcDrawImageString, vFXLppiipi)
-GO(XwcDrawString, vFXLppiipi)
-GO(XwcDrawText, vFXLpiibpiip_i)
+GOM(XwcDrawImageString, vFEXLppiipi)
+GOM(XwcDrawString, vFEXLppiipi)
+GOM(XwcDrawText, vFEXLpiipi)
 //GO(_XwcFreeStringList, 
 GO(XwcFreeStringList, vFbp_)
 //GO(_XwcGenericDrawImageString, 
@@ -1212,7 +1212,7 @@ GO(XwcFreeStringList, vFbp_)
 //GO(_XwcGenericTextEscapement, 
 //GO(_XwcGenericTextExtents, 
 //GO(_XwcGenericTextPerCharExtents, 
-//GO(XwcLookupString, iFpbiLiXLLLLiiiiuui_pibL_p)
+GOM(XwcLookupString, iFEpppipp)
 GO(XwcResetIC, pFp)
 //GO(_Xwcscmp, 
 //GO(_Xwcscpy, 
@@ -1220,13 +1220,13 @@ GO(XwcResetIC, pFp)
 //GO(_Xwcsncmp, 
 //GO(_Xwcsncpy, 
 //GO(_Xwcstombs, 
-GO(XwcTextEscapement, iFppi)
-GO(XwcTextExtents, iFppipp)
+GOM(XwcTextEscapement, iFEppi)
+GOM(XwcTextExtents, iFEppipp)
 //GO(_XwcTextListToTextProperty, 
-//GO(XwcTextListToTextProperty, iFXbp_iubpLiL_)
-GO(XwcTextPerCharExtents, iFppippippp)
+GOM(XwcTextListToTextProperty, iFEXpiup)
+GOM(XwcTextPerCharExtents, iFEppippippp)
 //GO(_XwcTextPropertyToTextList, 
-//GO(XwcTextPropertyToTextList, iFXbpLiL_bbp__p)
+GOM(XwcTextPropertyToTextList, iFEXbpLiL_pp)
 GO(_Xwctomb, iFpi)
 GO(XWhitePixel, LFXi)
 //GOM(XWhitePixelOfScreen, LFEbpXLiiiiipippLLLiiiil_)
