@@ -189,7 +189,7 @@ GO(SDL_GetClipboardText, pFv)
 GOM(SDL_GetClosestDisplayMode, pFipp)   //%noE
 //GO(SDL_GetColorKey, iFpp)
 GO(SDL_GetCPUCacheLineSize, iFv)
-GO(SDL_GetCPUCount, iFv)
+GOM(SDL_GetCPUCount, iFEv)
 GO(SDL_GetCurrentAudioDriver, pFv)
 GOM(SDL_GetCurrentDisplayMode, iFip)    //%noE
 GO(SDL_GetCurrentVideoDriver, pFv)
@@ -500,13 +500,13 @@ GOM(SDL_PushEvent, iFp)     //%noE
 GO(SDL_Quit, vFv)
  GO(SDL_QuitSubSystem, vFu)
 GO(SDL_RaiseWindow, vFp)
-//GOM(SDL_ReadBE16, uFEp)
-//GOM(SDL_ReadBE32, uFEp)
-//GOM(SDL_ReadBE64, UFEp)
-//GOM(SDL_ReadLE16, uFEp)
-//GOM(SDL_ReadLE32, uFEp)
-//GOM(SDL_ReadLE64, UFEp)
-//GOM(SDL_ReadU8, uFEp)
+GOM(SDL_ReadBE16, uFEp)
+GOM(SDL_ReadBE32, uFEp)
+GOM(SDL_ReadBE64, UFEp)
+GOM(SDL_ReadLE16, uFEp)
+GOM(SDL_ReadLE32, uFEp)
+GOM(SDL_ReadLE64, UFEp)
+GOM(SDL_ReadU8, uFEp)
 GO(SDL_realloc, pFpL)
 //GO(SDL_RecordGesture, iFl)
 //GO(SDL_RegisterEvents, uFi)
@@ -722,13 +722,13 @@ GO(SDL_WasInit, uFu)
 // SDL_wcslcat
 // SDL_wcslcpy
 //GO(SDL_wcslen, LFp)
-//GOM(SDL_WriteBE16, uFEpW)
-//GOM(SDL_WriteBE32, uFEpu)
-//GOM(SDL_WriteBE64, uFEpU)
-//GOM(SDL_WriteLE16, uFEpW)
-//GOM(SDL_WriteLE32, uFEpu)
-//GOM(SDL_WriteLE64, uFEpU)
-//GOM(SDL_WriteU8, uFEpu)
+GOM(SDL_WriteBE16, uFEpW)
+GOM(SDL_WriteBE32, uFEpu)
+GOM(SDL_WriteBE64, uFEpU)
+GOM(SDL_WriteLE16, uFEpW)
+GOM(SDL_WriteLE32, uFEpu)
+GOM(SDL_WriteLE64, uFEpU)
+GOM(SDL_WriteU8, uFEpu)
 
 GOM(SDL_RWseek, IFEpIi)
 GOM(SDL_RWtell, IFEp)
